@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/modrdn.c,v 1.27 2004/09/04 02:54:29 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/modrdn.c,v 1.30 2006/01/03 22:12:08 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -219,7 +219,7 @@ ldap_rename_s(
 		return rc;
 	}
 
-	rc = ldap_result( ld, msgid, 1, NULL, &res );
+	rc = ldap_result( ld, msgid, LDAP_MSG_ALL, NULL, &res );
 
 	if( rc == -1 ) {
 		return ld->ld_errno;
