@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/passwd.c,v 1.14 2004/04/06 23:11:32 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/passwd.c,v 1.18 2006/01/03 22:12:08 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,8 +36,7 @@ int ldap_parse_passwd(
 	struct berval *newpasswd )
 {
 	int rc;
-	char *retoid = NULL;
-	struct berval *retdata;
+	struct berval *retdata = NULL;
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
@@ -47,31 +46,32 @@ int ldap_parse_passwd(
 	newpasswd->bv_val = NULL;
 	newpasswd->bv_len = 0;
 
-	rc = ldap_parse_extended_result( ld, res, &retoid, &retdata, 0 );
-
-	if( rc != LDAP_SUCCESS ) {
+	rc = ldap_parse_extended_result( ld, res, NULL, &retdata, 0 );
+	if ( rc != LDAP_SUCCESS ) {
 		return rc;
 	}
 
-	if( retdata != NULL ) {
+	if ( retdata != NULL ) {
 		ber_tag_t tag;
 		BerElement *ber = ber_init( retdata );
 
-		if( ber == NULL ) {
-			ld->ld_errno = LDAP_NO_MEMORY;
-			return ld->ld_errno;
+		if ( ber == NULL ) {
+			rc = ld->ld_errno = LDAP_NO_MEMORY;
+			goto done;
 		}
 
 		/* we should check the tag */
 		tag = ber_scanf( ber, "{o}", newpasswd );
 		ber_free( ber, 1 );
 
-		if( tag == LBER_ERROR ) {
+		if ( tag == LBER_ERROR ) {
 			rc = ld->ld_errno = LDAP_DECODING_ERROR;
 		}
 	}
 
-	ber_memfree( retoid );
+done:;
+	ber_bvfree( retdata );
+
 	return rc;
 }
 
@@ -156,7 +156,7 @@ ldap_passwd_s(
 		return rc;
 	}
 
-	if ( ldap_result( ld, msgid, 1, (struct timeval *) NULL, &res ) == -1 ) {
+	if ( ldap_result( ld, msgid, LDAP_MSG_ALL, (struct timeval *) NULL, &res ) == -1 ) {
 		return ld->ld_errno;
 	}
 
