@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/add.c,v 1.208.2.3 2005/01/24 21:28:51 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/add.c,v 1.244 2006/03/30 15:59:26 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,17 +30,9 @@
 #include <ac/time.h>
 #include <ac/socket.h>
 
+#include "lutil.h"
 #include "slap.h"
 
-#ifdef LDAP_SLAPI
-#include "slapi/slapi.h"
-
-static void init_add_pblock( Operation *op, struct berval *dn, Entry *e,
-	int manageDSAit );
-static int call_add_preop_plugins( Operation *op );
-static void call_add_postop_plugins( Operation *op );
-#endif /* LDAP_SLAPI */
-
 int
 do_add( Operation *op, SlapReply *rs )
 {
@@ -49,13 +41,13 @@ do_add( Operation *op, SlapReply *rs )
 	struct berval	dn = BER_BVNULL;
 	ber_len_t	len;
 	ber_tag_t	tag;
-	Entry		*e;
 	Modifications	*modlist = NULL;
 	Modifications	**modtail = &modlist;
 	Modifications	tmp;
 	char		textbuf[ SLAP_TEXT_BUFLEN ];
 	size_t		textlen = sizeof( textbuf );
 	int		rc = 0;
+	int		freevals = 1;
 
 	Debug( LDAP_DEBUG_TRACE, "do_add\n", 0, 0, 0 );
 	/*
@@ -77,7 +69,7 @@ do_add( Operation *op, SlapReply *rs )
 		return SLAPD_DISCONNECT;
 	}
 
-	e = (Entry *) ch_calloc( 1, sizeof(Entry) );
+	op->ora_e = (Entry *) ch_calloc( 1, sizeof(Entry) );
 
 	rs->sr_err = dnPrettyNormal( NULL, &dn, &op->o_req_dn, &op->o_req_ndn,
 		op->o_tmpmemctx );
@@ -88,10 +80,10 @@ do_add( Operation *op, SlapReply *rs )
 		goto done;
 	}
 
-	ber_dupbv( &e->e_name, &op->o_req_dn );
-	ber_dupbv( &e->e_nname, &op->o_req_ndn );
+	ber_dupbv( &op->ora_e->e_name, &op->o_req_dn );
+	ber_dupbv( &op->ora_e->e_nname, &op->o_req_ndn );
 
-	Debug( LDAP_DEBUG_ARGS, "do_add: dn (%s)\n", e->e_dn, 0, 0 );
+	Debug( LDAP_DEBUG_ARGS, "do_add: dn (%s)\n", op->ora_e->e_dn, 0, 0 );
 
 	/* get the attrs */
 	for ( tag = ber_first_element( ber, &len, &last ); tag != LBER_DEFAULT;
@@ -121,6 +113,7 @@ do_add( Operation *op, SlapReply *rs )
 
 		mod  = (Modifications *) ch_malloc( sizeof(Modifications) );
 		mod->sml_op = LDAP_MOD_ADD;
+		mod->sml_flags = 0;
 		mod->sml_next = NULL;
 		mod->sml_desc = NULL;
 		mod->sml_type = tmp.sml_type;
@@ -150,22 +143,22 @@ do_add( Operation *op, SlapReply *rs )
 	}
 
 	Statslog( LDAP_DEBUG_STATS, "%s ADD dn=\"%s\"\n",
-	    op->o_log_prefix, e->e_name.bv_val, 0, 0, 0 );
+	    op->o_log_prefix, op->ora_e->e_name.bv_val, 0, 0, 0 );
 
-	if ( dn_match( &e->e_nname, &slap_empty_bv ) ) {
+	if ( dn_match( &op->ora_e->e_nname, &slap_empty_bv ) ) {
 		/* protocolError may be a more appropriate error */
 		send_ldap_error( op, rs, LDAP_ALREADY_EXISTS,
 			"root DSE already exists" );
 		goto done;
 
-	} else if ( dn_match( &e->e_nname, &frontendDB->be_schemandn ) ) {
+	} else if ( dn_match( &op->ora_e->e_nname, &frontendDB->be_schemandn ) ) {
 		send_ldap_error( op, rs, LDAP_ALREADY_EXISTS,
 			"subschema subentry already exists" );
 		goto done;
 	}
 
 	rs->sr_err = slap_mods_check( modlist, &rs->sr_text,
-			  textbuf, textlen, NULL );
+		textbuf, textlen, NULL );
 
 	if ( rs->sr_err != LDAP_SUCCESS ) {
 		send_ldap_result( op, rs );
@@ -173,23 +166,50 @@ do_add( Operation *op, SlapReply *rs )
 	}
 
 	/* temporary; remove if not invoking backend function */
-	op->ora_e = e;
 	op->ora_modlist = modlist;
 
+	/* call this so global overlays/SLAPI have access to ora_e */
+	rs->sr_err = slap_mods2entry( op->ora_modlist, &op->ora_e,
+		1, 0, &rs->sr_text, textbuf, textlen );
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		send_ldap_result( op, rs );
+		goto done;
+	}
+
+	freevals = 0;
+
 	op->o_bd = frontendDB;
 	rc = frontendDB->be_add( op, rs );
+
+#ifdef LDAP_X_TXN
+	if ( rc == LDAP_X_TXN_SPECIFY_OKAY ) {
+		/* skip cleanup */
+		return rc;
+	} else
+#endif
 	if ( rc == 0 ) {
-		e = NULL;
+		if ( op->ora_e != NULL && op->o_private != NULL ) {
+			BackendDB	*bd = op->o_bd;
+
+			op->o_bd = (BackendDB *)op->o_private;
+			op->o_private = NULL;
+
+			be_entry_release_w( op, op->ora_e );
+
+			op->ora_e = NULL;
+			op->o_bd = bd;
+			op->o_private = NULL;
+		}
 	}
 
 done:;
-	slap_graduate_commit_csn( op );
-
 	if ( modlist != NULL ) {
-		slap_mods_free( modlist );
+		/* in case of error, free the values as well */
+		slap_mods_free( modlist, freevals );
 	}
-	if ( e != NULL ) {
-		entry_free( e );
+
+	if ( op->ora_e != NULL ) {
+		entry_free( op->ora_e );
 	}
 	op->o_tmpfree( op->o_req_dn.bv_val, op->o_tmpmemctx );
 	op->o_tmpfree( op->o_req_ndn.bv_val, op->o_tmpmemctx );
@@ -201,10 +221,11 @@ int
 fe_op_add( Operation *op, SlapReply *rs )
 {
 	int		manageDSAit;
-	Entry		*e = op->ora_e;
-	Modifications	*modlist = op->ora_modlist;
-	Modifications	**modtail = &modlist;
+	Modifications	**modtail = &op->ora_modlist;
 	int		rc = 0;
+	BackendDB	*op_be, *bd = op->o_bd;
+	char		textbuf[ SLAP_TEXT_BUFLEN ];
+	size_t		textlen = sizeof( textbuf );
 
 	manageDSAit = get_manageDSAit( op );
 
@@ -213,29 +234,32 @@ fe_op_add( Operation *op, SlapReply *rs )
 	 * appropriate one, or send a referral to our "referral server"
 	 * if we don't hold it.
 	 */
-	op->o_bd = select_backend( &e->e_nname, manageDSAit, 1 );
+	op->o_bd = select_backend( &op->ora_e->e_nname, manageDSAit, 1 );
 	if ( op->o_bd == NULL ) {
+		op->o_bd = bd;
 		rs->sr_ref = referral_rewrite( default_referral,
-			NULL, &e->e_name, LDAP_SCOPE_DEFAULT );
+			NULL, &op->ora_e->e_name, LDAP_SCOPE_DEFAULT );
 		if ( !rs->sr_ref ) rs->sr_ref = default_referral;
 		if ( rs->sr_ref ) {
 			rs->sr_err = LDAP_REFERRAL;
-			op->o_bd = frontendDB;
 			send_ldap_result( op, rs );
-			op->o_bd = NULL;
 
 			if ( rs->sr_ref != default_referral ) {
 				ber_bvarray_free( rs->sr_ref );
 			}
 		} else {
-			op->o_bd = frontendDB;
 			send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 				"no global superior knowledge" );
-			op->o_bd = NULL;
 		}
 		goto done;
 	}
 
+	/* If we've got a glued backend, check the real backend */
+	op_be = op->o_bd;
+	if ( SLAP_GLUE_INSTANCE( op->o_bd )) {
+		op->o_bd = select_backend( &op->ora_e->e_nname, manageDSAit, 0 );
+	}
+
 	/* check restrictions */
 	if( backend_check_restrictions( op, rs, NULL ) != LDAP_SUCCESS ) {
 		send_ldap_result( op, rs );
@@ -247,9 +271,13 @@ fe_op_add( Operation *op, SlapReply *rs )
 		goto done;
 	}
 
-#ifdef LDAP_SLAPI
-	if ( op->o_pb ) init_add_pblock( op, &op->o_req_dn, e, manageDSAit );
-#endif /* LDAP_SLAPI */
+	rs->sr_err = slap_mods_obsolete_check( op, op->ora_modlist,
+		&rs->sr_text, textbuf, textlen );
+
+	if ( rs->sr_err != LDAP_SUCCESS ) {
+		send_ldap_result( op, rs );
+		goto done;
+	}
 
 	/*
 	 * do the add if 1 && (2 || 3)
@@ -260,19 +288,15 @@ fe_op_add( Operation *op, SlapReply *rs )
 	if ( op->o_bd->be_add ) {
 		/* do the update here */
 		int repl_user = be_isupdate( op );
-#ifndef SLAPD_MULTIMASTER
-		if ( !SLAP_SHADOW(op->o_bd) || repl_user )
-#endif
-		{
+		if ( !SLAP_SINGLE_SHADOW(op->o_bd) || repl_user ) {
 			int		update = !BER_BVISEMPTY( &op->o_bd->be_update_ndn );
-			char		textbuf[ SLAP_TEXT_BUFLEN ];
-			size_t		textlen = sizeof( textbuf );
 			slap_callback	cb = { NULL, slap_replog_cb, NULL, NULL };
 
+			op->o_bd = op_be;
+
 			if ( !update ) {
-				rs->sr_err = slap_mods_no_update_check( modlist,
-						&rs->sr_text,
-				  		textbuf, textlen );
+				rs->sr_err = slap_mods_no_user_mod_check( op, op->ora_modlist,
+					&rs->sr_text, textbuf, textlen );
 
 				if ( rs->sr_err != LDAP_SUCCESS ) {
 					send_ldap_result( op, rs );
@@ -282,7 +306,7 @@ fe_op_add( Operation *op, SlapReply *rs )
 
 			if ( !repl_user ) {
 				/* go to the last mod */
-				for ( modtail = &modlist;
+				for ( modtail = &op->ora_modlist;
 						*modtail != NULL;
 						modtail = &(*modtail)->sml_next )
 				{
@@ -290,78 +314,44 @@ fe_op_add( Operation *op, SlapReply *rs )
 					assert( (*modtail)->sml_desc != NULL );
 				}
 
-				rs->sr_err = slap_mods_opattrs( op, modlist,
-						modtail, &rs->sr_text,
-						textbuf, textlen, 1 );
+
+				/* check for duplicate values */
+				rs->sr_err = slap_mods_no_repl_user_mod_check( op,
+					op->ora_modlist, &rs->sr_text, textbuf, textlen );
 				if ( rs->sr_err != LDAP_SUCCESS ) {
 					send_ldap_result( op, rs );
 					goto done;
 				}
-			}
 
-			rs->sr_err = slap_mods2entry( modlist, &e, repl_user, 0,
-				&rs->sr_text, textbuf, textlen );
-			if ( rs->sr_err != LDAP_SUCCESS ) {
-				send_ldap_result( op, rs );
-				goto done;
-			}
-
-#ifdef LDAP_SLAPI
-			/*
-			 * Call the preoperation plugin here, because the entry
-			 * will actually contain something.
-			 */
-			if ( op->o_pb ) {
-				rs->sr_err = call_add_preop_plugins( op );
+				rs->sr_err = slap_mods2entry( *modtail, &op->ora_e,
+					0, 0, &rs->sr_text, textbuf, textlen );
 				if ( rs->sr_err != LDAP_SUCCESS ) {
-					/* plugin will have sent result */
+					send_ldap_result( op, rs );
 					goto done;
 				}
-			}
-#endif /* LDAP_SLAPI */
 
-			op->ora_e = e;
-#ifdef SLAPD_MULTIMASTER
-			if ( !repl_user )
-#endif
-			{
 				cb.sc_next = op->o_callback;
 				op->o_callback = &cb;
 			}
-			rc = (op->o_bd->be_add)( op, rs );
-			if ( rc == 0 ) {
-				/* FIXME: be_entry_release_w() should be
+
+			rc = op->o_bd->be_add( op, rs );
+			if ( rc == LDAP_SUCCESS ) {
+				/* NOTE: be_entry_release_w() is
 				 * called by do_add(), so that global
 				 * overlays on the way back can
 				 * at least read the entry */
-				be_entry_release_w( op, e );
-				e = NULL;
+				op->o_private = op->o_bd;
 			}
-			op->ora_e = NULL;
 
-#ifndef SLAPD_MULTIMASTER
 		} else {
 			BerVarray defref = NULL;
-#ifdef LDAP_SLAPI
-			/*
-			 * SLAPI_ADD_ENTRY will be empty, but this may be acceptable
-			 * on replicas (for now, it involves the minimum code intrusion).
-			 */
-			if ( op->o_pb ) {
-				rs->sr_err = call_add_preop_plugins( op );
-				if ( rs->sr_err != LDAP_SUCCESS ) {
-					/* plugin will have sent result */
-					goto done;
-				}
-			}
-#endif /* LDAP_SLAPI */
 
 			defref = op->o_bd->be_update_refs
 				? op->o_bd->be_update_refs : default_referral;
 
 			if ( defref != NULL ) {
 				rs->sr_ref = referral_rewrite( defref,
-					NULL, &e->e_name, LDAP_SCOPE_DEFAULT );
+					NULL, &op->ora_e->e_name, LDAP_SCOPE_DEFAULT );
 				if ( rs->sr_ref == NULL ) rs->sr_ref = defref;
 				rs->sr_err = LDAP_REFERRAL;
 				if (!rs->sr_ref) rs->sr_ref = default_referral;
@@ -375,28 +365,15 @@ fe_op_add( Operation *op, SlapReply *rs )
 					LDAP_UNWILLING_TO_PERFORM,
 					"shadow context; no update referral" );
 			}
-#endif /* SLAPD_MULTIMASTER */
 		}
 	} else {
-#ifdef LDAP_SLAPI
-		if ( op->o_pb ) {
-			rs->sr_err = call_add_preop_plugins( op );
-			if ( rs->sr_err != LDAP_SUCCESS ) {
-				/* plugin will have sent result */
-				goto done;
-			}
-		}
-#endif
-	    Debug( LDAP_DEBUG_ARGS, "	 do_add: no backend support\n", 0, 0, 0 );
-	    send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
+		Debug( LDAP_DEBUG_ARGS, "do_add: no backend support\n", 0, 0, 0 );
+		send_ldap_error( op, rs, LDAP_UNWILLING_TO_PERFORM,
 			"operation not supported within namingContext" );
 	}
 
-#ifdef LDAP_SLAPI
-	if ( op->o_pb ) call_add_postop_plugins( op );
-#endif /* LDAP_SLAPI */
-
 done:;
+	op->o_bd = bd;
 	return rc;
 }
 
@@ -404,22 +381,25 @@ int
 slap_mods2entry(
 	Modifications *mods,
 	Entry **e,
-	int repl_user,
+	int initial,
 	int dup,
 	const char **text,
 	char *textbuf, size_t textlen )
 {
-	Attribute **tail = &(*e)->e_attrs;
-	assert( *tail == NULL );
+	Attribute **tail;
+
+	if ( initial ) {
+		assert( (*e)->e_attrs == NULL );
+	}
+
+	for ( tail = &(*e)->e_attrs; *tail != NULL; tail = &(*tail)->a_next )
+		;
 
 	*text = textbuf;
 
 	for( ; mods != NULL; mods = mods->sml_next ) {
 		Attribute *attr;
 
-		if ( !repl_user ) {
-			assert( mods->sml_op == LDAP_MOD_ADD );
-		}
 		assert( mods->sml_desc != NULL );
 
 		attr = attr_find( (*e)->e_attrs, mods->sml_desc );
@@ -429,11 +409,15 @@ slap_mods2entry(
 #ifdef SLURPD_FRIENDLY
 			ber_len_t i,j;
 
-			if( !repl_user ) {
-				snprintf( textbuf, textlen,
-					"attribute '%s' provided more than once",
-					mods->sml_desc->ad_cname.bv_val );
-				return LDAP_TYPE_OR_VALUE_EXISTS;
+			if ( !initial ) {
+				/*	
+				 * This check allows overlays to override operational
+				 * attributes by setting them directly in the entry.
+				 * We assume slap_mods_no_user_mod_check() was called
+				 * with the user modifications.
+				 */
+				*text = NULL;
+				return LDAP_SUCCESS;
 			}
 
 			for( i=0; attr->a_vals[i].bv_val; i++ ) {
@@ -453,12 +437,11 @@ slap_mods2entry(
 				for ( j = 0; mods->sml_values[j].bv_val; j++ ) {
 					ber_dupbv( &attr->a_vals[i+j], &mods->sml_values[j] );
 				}
-				BER_BVZERO( &attr->a_vals[i+j] );	
+				BER_BVZERO( &attr->a_vals[i+j] );
+				j++;
 			} else {
 				AC_MEMCPY( &attr->a_vals[i], mods->sml_values,
 					sizeof( struct berval ) * j );
-				ch_free( mods->sml_values );
-				mods->sml_values = NULL;
 			}
 
 			if( mods->sml_nvalues ) {
@@ -472,8 +455,6 @@ slap_mods2entry(
 				} else {
 					AC_MEMCPY( &attr->a_nvals[i], mods->sml_nvalues,
 						sizeof( struct berval ) * j );
-					ch_free( mods->sml_nvalues );
-					mods->sml_nvalues = NULL;
 				}
 			} else {
 				attr->a_nvals = attr->a_vals;
@@ -484,62 +465,42 @@ slap_mods2entry(
 			snprintf( textbuf, textlen,
 				"attribute '%s' provided more than once",
 				mods->sml_desc->ad_cname.bv_val );
+			*text = textbuf;
 			return LDAP_TYPE_OR_VALUE_EXISTS;
 #endif
 		}
 
 		if( mods->sml_values[1].bv_val != NULL ) {
 			/* check for duplicates */
-			int		i, j;
+			int		i, j, rc, match;
 			MatchingRule *mr = mods->sml_desc->ad_type->sat_equality;
 
-			/* check if the values we're adding already exist */
-			if( mr == NULL || !mr->smr_match ) {
-				for ( i = 1; mods->sml_values[i].bv_val != NULL; i++ ) {
-					/* test asserted values against themselves */
-					for( j = 0; j < i; j++ ) {
-						if ( bvmatch( &mods->sml_values[i],
-							&mods->sml_values[j] ) )
-						{
-							/* value exists already */
-							snprintf( textbuf, textlen,
-								"%s: value #%d provided more than once",
-								mods->sml_desc->ad_cname.bv_val, j );
-							return LDAP_TYPE_OR_VALUE_EXISTS;
-						}
-					}
-				}
+			for ( i = 1; mods->sml_values[i].bv_val != NULL; i++ ) {
+				/* test asserted values against themselves */
+				for( j = 0; j < i; j++ ) {
+					rc = ordered_value_match( &match, mods->sml_desc, mr,
+						SLAP_MR_EQUALITY
+						| SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX
+						| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH
+						| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH,
+						mods->sml_nvalues
+							? &mods->sml_nvalues[i]
+							: &mods->sml_values[i],
+						mods->sml_nvalues
+							? &mods->sml_nvalues[j]
+							: &mods->sml_values[j],
+						text );
 
-			} else {
-				int	rc;
-				int match;
+					if ( rc == LDAP_SUCCESS && match == 0 ) {
+						/* value exists already */
+						snprintf( textbuf, textlen,
+							"%s: value #%d provided more than once",
+							mods->sml_desc->ad_cname.bv_val, j );
+						*text = textbuf;
+						return LDAP_TYPE_OR_VALUE_EXISTS;
 
-				for ( i = 1; mods->sml_values[i].bv_val != NULL; i++ ) {
-					/* test asserted values against themselves */
-					for( j = 0; j < i; j++ ) {
-						rc = value_match( &match, mods->sml_desc, mr,
-							SLAP_MR_EQUALITY
-							| SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX
-							| SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH
-							| SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH,
-							mods->sml_nvalues
-								? &mods->sml_nvalues[i]
-								: &mods->sml_values[i],
-							mods->sml_nvalues
-								? &mods->sml_nvalues[j]
-								: &mods->sml_values[j],
-							text );
-
-						if ( rc == LDAP_SUCCESS && match == 0 ) {
-							/* value exists already */
-							snprintf( textbuf, textlen,
-								"%s: value #%d provided more than once",
-								mods->sml_desc->ad_cname.bv_val, j );
-							return LDAP_TYPE_OR_VALUE_EXISTS;
-
-						} else if ( rc != LDAP_SUCCESS ) {
-							return rc;
-						}
+					} else if ( rc != LDAP_SUCCESS ) {
+						return rc;
 					}
 				}
 			}
@@ -549,7 +510,6 @@ slap_mods2entry(
 
 		/* move ad to attr structure */
 		attr->a_desc = mods->sml_desc;
-		if ( !dup ) mods->sml_desc = NULL;
 
 		/* move values to attr structure */
 		/*	should check for duplicates */
@@ -563,7 +523,6 @@ slap_mods2entry(
 			BER_BVZERO( &attr->a_vals[i] );
 		} else {
 			attr->a_vals = mods->sml_values;
-			mods->sml_values = NULL;
 		}
 
 		if ( mods->sml_nvalues ) {
@@ -577,7 +536,6 @@ slap_mods2entry(
 				BER_BVZERO( &attr->a_nvals[i] );
 			} else {
 				attr->a_nvals = mods->sml_nvalues;
-				mods->sml_nvalues = NULL;
 			}
 		} else {
 			attr->a_nvals = attr->a_vals;
@@ -613,6 +571,7 @@ slap_entry2mods(
 		mod = (Modifications *) malloc( sizeof( Modifications ));
 		
 		mod->sml_op = LDAP_MOD_REPLACE;
+		mod->sml_flags = 0;
 
 		mod->sml_type = a_new_desc->ad_cname;
 
@@ -659,51 +618,126 @@ slap_entry2mods(
 	return LDAP_SUCCESS;
 }
 
-#ifdef LDAP_SLAPI
-static void init_add_pblock( Operation *op,
-	struct berval *dn, Entry *e, int manageDSAit )
+int slap_add_opattrs(
+	Operation *op,
+	const char **text,
+	char *textbuf,
+	size_t textlen,
+	int manage_ctxcsn )
 {
-	slapi_int_pblock_set_operation( op->o_pb, op );
-	slapi_pblock_set( op->o_pb, SLAPI_ADD_TARGET, (void *)dn->bv_val );
-	slapi_pblock_set( op->o_pb, SLAPI_ADD_ENTRY, (void *)e );
-	slapi_pblock_set( op->o_pb, SLAPI_MANAGEDSAIT, (void *)manageDSAit );
-}
+	struct berval name, timestamp, csn = BER_BVNULL;
+	struct berval nname, tmp;
+	char timebuf[ LDAP_LUTIL_GENTIME_BUFSIZE ];
+	char csnbuf[ LDAP_LUTIL_CSNSTR_BUFSIZE ];
+	Attribute *a;
 
-static int call_add_preop_plugins( Operation *op )
-{
-	int rc;
+	a = attr_find( op->ora_e->e_attrs,
+		slap_schema.si_ad_structuralObjectClass );
 
-	rc = slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_PRE_ADD_FN, op->o_pb );
-	if ( rc < 0 ) {
-		/*
-		 * A preoperation plugin failure will abort the
-		 * entire operation.
-		 */
-		Debug(LDAP_DEBUG_TRACE,
-			"do_add: add preoperation plugin failed.\n",
-			0, 0, 0);
+	if ( !a ) {
+		Attribute *oc;
+		int rc;
 
-		if (( slapi_pblock_get( op->o_pb, SLAPI_RESULT_CODE,
-			(void *)&rc ) != 0 ) || rc == LDAP_SUCCESS )
-		{
-			rc = LDAP_OTHER;
+		oc = attr_find( op->ora_e->e_attrs, slap_schema.si_ad_objectClass );
+		if ( oc ) {
+			rc = structural_class( oc->a_vals, &tmp, NULL, text,
+				textbuf, textlen );
+			if( rc != LDAP_SUCCESS ) return rc;
+
+			attr_merge_one( op->ora_e, slap_schema.si_ad_structuralObjectClass,
+				&tmp, NULL );
 		}
-	} else {
-		rc = LDAP_SUCCESS;
 	}
 
-	return rc;
-}
+	if ( SLAP_LASTMOD( op->o_bd ) ) {
+		char *ptr;
+		int gotcsn = 0;
 
-static void call_add_postop_plugins( Operation *op )
-{
-	int rc;
+		timestamp.bv_val = timebuf;
+		a = attr_find( op->ora_e->e_attrs, slap_schema.si_ad_entryCSN );
+		if ( a ) {
+			gotcsn = 1;
+			csn = a->a_vals[0];
+		}
+		if ( BER_BVISEMPTY( &op->o_csn )) {
+			if ( !gotcsn ) {
+				csn.bv_val = csnbuf;
+				csn.bv_len = sizeof(csnbuf);
+				slap_get_csn( op, &csn, manage_ctxcsn );
+			} else {
+				if ( manage_ctxcsn )
+					slap_queue_csn( op, &csn );
+			}
+		} else {
+			csn = op->o_csn;
+		}
+		ptr = ber_bvchr( &csn, '#' );
+		if ( ptr ) {
+			timestamp.bv_len = ptr - csn.bv_val;
+			if ( timestamp.bv_len >= sizeof(timebuf) )	/* ?!? */
+				timestamp.bv_len = sizeof(timebuf) - 1;
+			AC_MEMCPY( timebuf, csn.bv_val, timestamp.bv_len );
+			timebuf[timestamp.bv_len] = '\0';
+		} else {
+			time_t now = slap_get_time();
 
-	rc = slapi_int_call_plugins( op->o_bd, SLAPI_PLUGIN_POST_ADD_FN, op->o_pb );
-	if ( rc < 0 ) {
-		Debug(LDAP_DEBUG_TRACE,
-			"do_add: add postoperation plugin failed\n",
-			0, 0, 0);
+			timestamp.bv_len = sizeof(timebuf);
+
+			slap_timestamp( &now, &timestamp );
+		}
+
+		if ( BER_BVISEMPTY( &op->o_dn ) ) {
+			BER_BVSTR( &name, SLAPD_ANONYMOUS );
+			nname = name;
+		} else {
+			name = op->o_dn;
+			nname = op->o_ndn;
+		}
+
+		a = attr_find( op->ora_e->e_attrs,
+			slap_schema.si_ad_entryUUID );
+		if ( !a ) {
+			char uuidbuf[ LDAP_LUTIL_UUIDSTR_BUFSIZE ];
+
+			tmp.bv_len = lutil_uuidstr( uuidbuf, sizeof( uuidbuf ) );
+			tmp.bv_val = uuidbuf;
+			
+			attr_merge_normalize_one( op->ora_e,
+				slap_schema.si_ad_entryUUID, &tmp, op->o_tmpmemctx );
+		}
+
+		a = attr_find( op->ora_e->e_attrs,
+			slap_schema.si_ad_creatorsName );
+		if ( !a ) {
+			attr_merge_one( op->ora_e,
+				slap_schema.si_ad_creatorsName, &name, &nname );
+		}
+
+		a = attr_find( op->ora_e->e_attrs,
+			slap_schema.si_ad_createTimestamp );
+		if ( !a ) {
+			attr_merge_one( op->ora_e,
+				slap_schema.si_ad_createTimestamp, &timestamp, NULL );
+		}
+
+		if ( !gotcsn ) {
+			attr_merge_one( op->ora_e,
+				slap_schema.si_ad_entryCSN, &csn, NULL );
+		}
+
+		a = attr_find( op->ora_e->e_attrs,
+			slap_schema.si_ad_modifiersName );
+		if ( !a ) {
+			attr_merge_one( op->ora_e,
+				slap_schema.si_ad_modifiersName, &name, &nname );
+		}
+
+		a = attr_find( op->ora_e->e_attrs,
+			slap_schema.si_ad_modifyTimestamp );
+		if ( !a ) {
+			attr_merge_one( op->ora_e,
+				slap_schema.si_ad_modifyTimestamp, &timestamp, NULL );
+		}
 	}
+	return LDAP_SUCCESS;
 }
-#endif /* LDAP_SLAPI */
