@@ -1,8 +1,8 @@
 /* id2entry.c - routines to deal with the id2entry database */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/id2entry.c,v 1.62.2.2 2005/01/20 18:03:58 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/id2entry.c,v 1.72 2006/03/31 10:53:32 hyc Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2005 The OpenLDAP Foundation.
+ * Copyright 2000-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -92,14 +92,16 @@ int bdb_id2entry_update(
 int bdb_id2entry(
 	BackendDB *be,
 	DB_TXN *tid,
+	u_int32_t locker,
 	ID id,
 	Entry **e )
 {
 	struct bdb_info *bdb = (struct bdb_info *) be->be_private;
 	DB *db = bdb->bi_id2entry->bdi_db;
 	DBT key, data;
+	DBC *cursor;
 	struct berval bv;
-	int rc = 0, ret = 0;
+	int rc = 0;
 	ID nid;
 
 	*e = NULL;
@@ -113,7 +115,15 @@ int bdb_id2entry(
 	data.flags = DB_DBT_MALLOC;
 
 	/* fetch it */
-	rc = db->get( db, tid, &key, &data, bdb->bi_db_opflags );
+	rc = db->cursor( db, tid, &cursor, bdb->bi_db_opflags );
+	if ( rc ) return rc;
+
+	/* Use our own locker if needed */
+	if ( !tid && locker )
+		cursor->locker = locker;
+
+	rc = cursor->c_get( cursor, &key, &data, DB_SET );
+	cursor->c_close( cursor );
 
 	if( rc != 0 ) {
 		return rc;
@@ -212,14 +222,10 @@ int bdb_entry_return(
 		e->e_nname.bv_val = NULL;
 	}
 #ifndef SLAP_ZONE_ALLOC
-#ifndef BDB_HIER
 	/* In tool mode the e_bv buffer is realloc'd, leave it alone */
 	if( !(slapMode & SLAP_TOOL_MODE) ) {
 		free( e->e_bv.bv_val );
 	}
-#else
-	free( e->e_bv.bv_val );
-#endif /* BDB_HIER */
 #endif /* !SLAP_ZONE_ALLOC */
 
 #ifdef SLAP_ZONE_ALLOC
@@ -252,9 +258,8 @@ int bdb_entry_release(
 #endif
 		}
 		/* free entry and reader or writer lock */
-		if ( op ) {
-			boi = (struct bdb_op_info *)op->o_private;
-		}
+		boi = (struct bdb_op_info *)op->o_private;
+
 		/* lock is freed with txn */
 		if ( !boi || boi->boi_txn ) {
 			bdb_unlocked_cache_return_entry_rw( &bdb->bi_cache, e, rw );
@@ -399,8 +404,8 @@ dn2entry_retry:
 
 	if ( oc && !is_entry_objectclass( e, oc, 0 )) {
 		Debug( LDAP_DEBUG_ACL,
-			"<= bdb_entry_get: failed to find objectClass\n",
-			0, 0, 0 ); 
+			"<= bdb_entry_get: failed to find objectClass %s\n",
+			oc->soc_cname.bv_val, 0, 0 ); 
 		rc = LDAP_NO_SUCH_ATTRIBUTE;
 		goto return_results;
 	}
