@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/bind.c,v 1.95.2.3 2006/05/15 17:04:42 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-meta/bind.c,v 1.95.2.4 2006/05/17 15:27:21 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2006 The OpenLDAP Foundation.
@@ -636,13 +636,21 @@ retry:;
 
 done:;
 	rs->sr_err = rc;
-	if ( rc != LDAP_SUCCESS && META_BACK_ONERR_STOP( mi ) ) {
+	if ( rc != LDAP_SUCCESS ) {
+		if ( dolock ) {
+			ldap_pvt_thread_mutex_lock( &mi->mi_conninfo.lai_mutex );
+		}
 	        LDAP_BACK_CONN_BINDING_CLEAR( msc );
-	        LDAP_BACK_CONN_TAINTED_SET( mc );
-		meta_back_release_conn_lock( op, mc, dolock );
-		*mcp = NULL;
+		if ( META_BACK_ONERR_STOP( mi ) ) {
+	        	LDAP_BACK_CONN_TAINTED_SET( mc );
+			meta_back_release_conn_lock( op, mc, dolock );
+			*mcp = NULL;
+		}
+		if ( dolock ) {
+			ldap_pvt_thread_mutex_unlock( &mi->mi_conninfo.lai_mutex );
+		}
 
-		if ( sendok & LDAP_BACK_SENDERR ) {
+		if ( META_BACK_ONERR_STOP( mi ) && ( sendok & LDAP_BACK_SENDERR ) ) {
 			send_ldap_result( op, rs );
 		}
 	}
