@@ -1,8 +1,9 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/init.c,v 1.47 2004/11/28 23:50:55 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/init.c,v 1.73 2006/01/03 22:12:24 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2004 The OpenLDAP Foundation.
+ * Copyright 1999-2006 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
+ * Portions Copyright 2002 Pierangelo Masarati.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -15,7 +16,8 @@
  */
 /* ACKNOWLEDGEMENTS:
  * This work was initially developed by Dmitry Kovalev for inclusion
- * by OpenLDAP Software.
+ * by OpenLDAP Software.  Additional significant contributors include
+ * Pierangelo Masarati.
  */
 
 #include "portable.h"
@@ -32,19 +34,25 @@ sql_back_initialize(
 	BackendInfo	*bi )
 { 
 	static char *controls[] = {
+		LDAP_CONTROL_ASSERT,
+		LDAP_CONTROL_MANAGEDSAIT,
 #if 0 /* needs improvements */
-#ifdef LDAP_CONTROL_NOOP
 		LDAP_CONTROL_NOOP,
-#endif /* LDAP_CONTROL_NOOP */
 #endif
-#ifdef LDAP_CONTROL_VALUESRETURNFILTER
- 		LDAP_CONTROL_VALUESRETURNFILTER,
-#endif /* LDAP_CONTROL_VALUESRETURNFILTER */
+#ifdef SLAP_CONTROL_X_TREE_DELETE
+		SLAP_CONTROL_X_TREE_DELETE,
+#endif /* SLAP_CONTROL_X_TREE_DELETE */
 		NULL
 	};
 
 	bi->bi_controls = controls;
 
+	bi->bi_flags |=
+#if 0
+		SLAP_BFLAG_INCREMENT |
+#endif
+		SLAP_BFLAG_REFERRALS;
+
 	Debug( LDAP_DEBUG_TRACE,"==>sql_back_initialize()\n", 0, 0, 0 );
 	
 	bi->bi_db_init = backsql_db_init;
@@ -66,6 +74,7 @@ sql_back_initialize(
 	bi->bi_chk_referrals = 0;
 	bi->bi_operational = backsql_operational;
 	bi->bi_entry_get_rw = backsql_entry_get;
+	bi->bi_entry_release_rw = backsql_entry_release;
  
 	bi->bi_connection_init = 0;
 	bi->bi_connection_destroy = backsql_connection_destroy;
@@ -88,17 +97,23 @@ backsql_db_init(
 	BackendDB 	*bd )
 {
 	backsql_info	*bi;
+	int		rc = 0;
  
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_db_init()\n", 0, 0, 0 );
+
 	bi = (backsql_info *)ch_calloc( 1, sizeof( backsql_info ) );
-	memset( bi, '\0', sizeof( backsql_info ) );
 	ldap_pvt_thread_mutex_init( &bi->sql_dbconn_mutex );
 	ldap_pvt_thread_mutex_init( &bi->sql_schema_mutex );
-	backsql_init_db_env( bi );
+
+	if ( backsql_init_db_env( bi ) != SQL_SUCCESS ) {
+		rc = -1;
+	}
 
 	bd->be_private = bi;
+
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_db_init()\n", 0, 0, 0 );
-	return 0;
+
+	return rc;
 }
 
 int
@@ -108,6 +123,7 @@ backsql_db_destroy(
 	backsql_info	*bi = (backsql_info*)bd->be_private;
  
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_db_destroy()\n", 0, 0, 0 );
+
 	ldap_pvt_thread_mutex_lock( &bi->sql_dbconn_mutex );
 	backsql_free_db_env( bi );
 	ldap_pvt_thread_mutex_unlock( &bi->sql_dbconn_mutex );
@@ -116,33 +132,85 @@ backsql_db_destroy(
 	backsql_destroy_schema_map( bi );
 	ldap_pvt_thread_mutex_unlock( &bi->sql_schema_mutex );
 	ldap_pvt_thread_mutex_destroy( &bi->sql_schema_mutex );
-	free( bi->sql_dbname );
-	free( bi->sql_dbuser );
+
+	if ( bi->sql_dbname ) {
+		ch_free( bi->sql_dbname );
+	}
+	if ( bi->sql_dbuser ) {
+		ch_free( bi->sql_dbuser );
+	}
 	if ( bi->sql_dbpasswd ) {
-		free( bi->sql_dbpasswd );
+		ch_free( bi->sql_dbpasswd );
 	}
 	if ( bi->sql_dbhost ) {
-		free( bi->sql_dbhost );
+		ch_free( bi->sql_dbhost );
 	}
 	if ( bi->sql_upper_func.bv_val ) {
-		free( bi->sql_upper_func.bv_val );
-		free( bi->sql_upper_func_open.bv_val );
-		free( bi->sql_upper_func_close.bv_val );
+		ch_free( bi->sql_upper_func.bv_val );
+		ch_free( bi->sql_upper_func_open.bv_val );
+		ch_free( bi->sql_upper_func_close.bv_val );
+	}
+	if ( bi->sql_concat_func ) {
+		ber_bvarray_free( bi->sql_concat_func );
+	}
+	if ( !BER_BVISNULL( &bi->sql_strcast_func ) ) {
+		ch_free( bi->sql_strcast_func.bv_val );
+	}
+	if ( !BER_BVISNULL( &bi->sql_children_cond ) ) {
+		ch_free( bi->sql_children_cond.bv_val );
+	}
+	if ( !BER_BVISNULL( &bi->sql_subtree_cond ) ) {
+		ch_free( bi->sql_subtree_cond.bv_val );
+	}
+	if ( !BER_BVISNULL( &bi->sql_dn_oc_aliasing ) ) {
+		ch_free( bi->sql_dn_oc_aliasing.bv_val );
+	}
+	if ( bi->sql_oc_query ) {
+		ch_free( bi->sql_oc_query );
+	}
+	if ( bi->sql_at_query ) {
+		ch_free( bi->sql_at_query );
+	}
+	if ( bi->sql_id_query ) {
+		ch_free( bi->sql_id_query );
+	}
+	if ( bi->sql_has_children_query ) {
+		ch_free( bi->sql_has_children_query );
+	}
+	if ( bi->sql_insentry_stmt ) {
+		ch_free( bi->sql_insentry_stmt );
+	}
+	if ( bi->sql_delentry_stmt ) {
+		ch_free( bi->sql_delentry_stmt );
+	}
+	if ( bi->sql_renentry_stmt ) {
+		ch_free( bi->sql_renentry_stmt );
+	}
+	if ( bi->sql_delobjclasses_stmt ) {
+		ch_free( bi->sql_delobjclasses_stmt );
+	}
+	if ( !BER_BVISNULL( &bi->sql_aliasing ) ) {
+		ch_free( bi->sql_aliasing.bv_val );
+	}
+	if ( !BER_BVISNULL( &bi->sql_aliasing_quote ) ) {
+		ch_free( bi->sql_aliasing_quote.bv_val );
+	}
+
+	if ( bi->sql_anlist ) {
+		int	i;
+
+		for ( i = 0; !BER_BVISNULL( &bi->sql_anlist[ i ].an_name ); i++ )
+		{
+			ch_free( bi->sql_anlist[ i ].an_name.bv_val );
+		}
+		ch_free( bi->sql_anlist );
 	}
-	
-	free( bi->sql_subtree_cond.bv_val );
-	free( bi->sql_oc_query );
-	free( bi->sql_at_query );
-	free( bi->sql_insentry_query );
-	free( bi->sql_delentry_query );
-	free( bi->sql_delobjclasses_query );
-	free( bi->sql_delreferrals_query );
 
 	if ( bi->sql_baseObject ) {
 		entry_free( bi->sql_baseObject );
 	}
 	
-	free( bi );
+	ch_free( bi );
 	
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_db_destroy()\n", 0, 0, 0 );
 	return 0;
@@ -153,12 +221,11 @@ backsql_db_open(
 	BackendDB 	*bd )
 {
 	backsql_info 	*bi = (backsql_info*)bd->be_private;
-	SQLHDBC 	dbh;
-	ber_len_t	idq_len;
+	SQLHDBC 	dbh = SQL_NULL_HDBC;
 	struct berbuf	bb = BB_NULL;
 
-	char		opbuf[ OPERATION_BUFFER_SIZE ];
-	Operation*	op = (Operation *)opbuf;
+	OperationBuffer	opbuf;
+	Operation*	op = (Operation *) &opbuf;
 	
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_db_open(): "
 		"testing RDBMS connection\n", 0, 0, 0 );
@@ -183,6 +250,21 @@ backsql_db_open(
 		}
 	}
 
+	/*
+	 * see back-sql.h for default values
+	 */
+	if ( BER_BVISNULL( &bi->sql_aliasing ) ) {
+		ber_str2bv( BACKSQL_ALIASING,
+			STRLENOF( BACKSQL_ALIASING ),
+			1, &bi->sql_aliasing );
+	}
+
+	if ( BER_BVISNULL( &bi->sql_aliasing_quote ) ) {
+		ber_str2bv( BACKSQL_ALIASING_QUOTE,
+			STRLENOF( BACKSQL_ALIASING_QUOTE ),
+			1, &bi->sql_aliasing_quote );
+	}
+
 	/*
 	 * Prepare cast string as required
 	 */
@@ -211,10 +293,10 @@ backsql_db_open(
 
 	/* normalize filter values only if necessary */
 	bi->sql_caseIgnoreMatch = mr_find( "caseIgnoreMatch" );
-	assert( bi->sql_caseIgnoreMatch );
+	assert( bi->sql_caseIgnoreMatch != NULL );
 
 	bi->sql_telephoneNumberMatch = mr_find( "telephoneNumberMatch" );
-	assert( bi->sql_telephoneNumberMatch );
+	assert( bi->sql_telephoneNumberMatch != NULL );
 
 	if ( bi->sql_dbuser == NULL ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
@@ -223,7 +305,7 @@ backsql_db_open(
 		return 1;
 	}
 	
-	if ( bi->sql_subtree_cond.bv_val == NULL ) {
+	if ( BER_BVISNULL( &bi->sql_subtree_cond ) ) {
 		/*
 		 * Prepare concat function for subtree search condition
 		 */
@@ -253,7 +335,7 @@ backsql_db_open(
 			 * UPPER(ldap_entries.dn) LIKE UPPER(CONCAT('%',?))
 			 */
 
-			backsql_strfcat( &bb, "blbbb",
+			backsql_strfcat_x( &bb, NULL, "blbbb",
 					&bi->sql_upper_func,
 					(ber_len_t)STRLENOF( "(ldap_entries.dn) LIKE " ),
 						"(ldap_entries.dn) LIKE ",
@@ -267,12 +349,14 @@ backsql_db_open(
 			 * ldap_entries.dn LIKE CONCAT('%',?)
 			 */
 
-			backsql_strfcat( &bb, "lb",
+			backsql_strfcat_x( &bb, NULL, "lb",
 					(ber_len_t)STRLENOF( "ldap_entries.dn LIKE " ),
 						"ldap_entries.dn LIKE ",
 					&concat );
 		}
 
+		ch_free( concat.bv_val );
+
 		bi->sql_subtree_cond = bb.bb_val;
 			
 		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
@@ -289,7 +373,7 @@ backsql_db_open(
 			 * UPPER(ldap_entries.dn) LIKE UPPER(CONCAT('%,',?))
 			 */
 
-			backsql_strfcat( &bb, "blbl",
+			backsql_strfcat_x( &bb, NULL, "blbl",
 					&bi->sql_upper_func,
 					(ber_len_t)STRLENOF( "(ldap_entries.dn)=" ),
 						"(ldap_entries.dn)=",
@@ -302,7 +386,7 @@ backsql_db_open(
 			 * ldap_entries.dn LIKE CONCAT('%,',?)
 			 */
 
-			backsql_strfcat( &bb, "l",
+			backsql_strfcat_x( &bb, NULL, "l",
 					(ber_len_t)STRLENOF( "ldap_entries.dn=?" ),
 						"ldap_entries.dn=?");
 		}
@@ -342,50 +426,51 @@ backsql_db_open(
 		bi->sql_at_query = ch_strdup( backsql_def_at_query );
 	}
 	
-	if ( bi->sql_insentry_query == NULL ) {
+	if ( bi->sql_insentry_stmt == NULL ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
 			"entry insertion SQL statement not specified "
-			"(use \"insentry_query\" directive in slapd.conf)\n",
+			"(use \"insentry_stmt\" directive in slapd.conf)\n",
 			0, 0, 0 );
 		Debug(LDAP_DEBUG_TRACE, "backsql_db_open(): "
 			"setting \"%s\" by default\n",
-			backsql_def_insentry_query, 0, 0 );
-		bi->sql_insentry_query = ch_strdup( backsql_def_insentry_query );
+			backsql_def_insentry_stmt, 0, 0 );
+		bi->sql_insentry_stmt = ch_strdup( backsql_def_insentry_stmt );
 	}
 	
-	if ( bi->sql_delentry_query == NULL ) {
+	if ( bi->sql_delentry_stmt == NULL ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
 			"entry deletion SQL statement not specified "
-			"(use \"delentry_query\" directive in slapd.conf)\n",
+			"(use \"delentry_stmt\" directive in slapd.conf)\n",
 			0, 0, 0 );
 		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
 			"setting \"%s\" by default\n",
-			backsql_def_delentry_query, 0, 0 );
-		bi->sql_delentry_query = ch_strdup( backsql_def_delentry_query );
+			backsql_def_delentry_stmt, 0, 0 );
+		bi->sql_delentry_stmt = ch_strdup( backsql_def_delentry_stmt );
 	}
 
-	if ( bi->sql_delobjclasses_query == NULL ) {
+	if ( bi->sql_renentry_stmt == NULL ) {
+		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
+			"entry deletion SQL statement not specified "
+			"(use \"renentry_stmt\" directive in slapd.conf)\n",
+			0, 0, 0 );
+		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
+			"setting \"%s\" by default\n",
+			backsql_def_renentry_stmt, 0, 0 );
+		bi->sql_renentry_stmt = ch_strdup( backsql_def_renentry_stmt );
+	}
+
+	if ( bi->sql_delobjclasses_stmt == NULL ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
 			"objclasses deletion SQL statement not specified "
-			"(use \"delobjclasses_query\" directive in slapd.conf)\n",
+			"(use \"delobjclasses_stmt\" directive in slapd.conf)\n",
 			0, 0, 0 );
 		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
 			"setting \"%s\" by default\n",
-			backsql_def_delobjclasses_query, 0, 0 );
-		bi->sql_delobjclasses_query = ch_strdup( backsql_def_delobjclasses_query );
-	}
-
-	if ( bi->sql_delreferrals_query == NULL ) {
-		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
-			"referrals deletion SQL statement not specified "
-			"(use \"delreferrals_query\" directive in slapd.conf)\n",
-			0, 0, 0 );
-		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
-			"setting \"%s\" by default\n",
-			backsql_def_delreferrals_query, 0, 0 );
-		bi->sql_delreferrals_query = ch_strdup( backsql_def_delreferrals_query );
+			backsql_def_delobjclasses_stmt, 0, 0 );
+		bi->sql_delobjclasses_stmt = ch_strdup( backsql_def_delobjclasses_stmt );
 	}
 
+	/* This should just be to force schema loading */
 	op->o_hdr = (Opheader *)&op[ 1 ];
 	op->o_connid = (unsigned long)(-1);
 	op->o_bd = bd;
@@ -395,59 +480,83 @@ backsql_db_open(
 		return 1;
 	}
 
-	/*
-	 * Prepare ID selection query
-	 */
-	bi->sql_id_query = NULL;
-	idq_len = 0;
-
-	if ( bi->sql_upper_func.bv_val == NULL ) {
-		backsql_strcat( &bb, backsql_id_query, "dn=?", NULL );
-
-	} else {
-		if ( BACKSQL_HAS_LDAPINFO_DN_RU( bi ) ) {
-			backsql_strcat( &bb, backsql_id_query,
-					"dn_ru=?", NULL );
-		} else {
-			if ( BACKSQL_USE_REVERSE_DN( bi ) ) {
-				backsql_strfcat( &bb, "sbl",
-						backsql_id_query,
-						&bi->sql_upper_func, 
-						(ber_len_t)STRLENOF( "(dn)=?" ), "(dn)=?" );
-			} else {
-				backsql_strfcat( &bb, "sblbcb",
-						backsql_id_query,
-						&bi->sql_upper_func, 
-						(ber_len_t)STRLENOF( "(dn)=" ), "(dn)=",
-						&bi->sql_upper_func_open, 
-						'?', 
-						&bi->sql_upper_func_close );
-			}
-		}
+	if ( backsql_free_db_conn( op ) != SQL_SUCCESS ) {
+		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
+			"connection free failed\n", 0, 0, 0 );
 	}
-	bi->sql_id_query = bb.bb_val.bv_val;
-
-       	/*
-	 * Prepare children ID selection query
-	 */
-	bi->sql_has_children_query = NULL;
-
-	bb.bb_val.bv_val = NULL;
-	bb.bb_val.bv_len = 0;
-	bb.bb_len = 0;
-	backsql_strfcat( &bb, "sb",
-			"SELECT COUNT(distinct subordinates.id) FROM ldap_entries,ldap_entries subordinates WHERE subordinates.parent=ldap_entries.id AND ",
-
-			&bi->sql_children_cond );
-	bi->sql_has_children_query = bb.bb_val.bv_val;
- 
-	backsql_free_db_conn( op );
 	if ( !BACKSQL_SCHEMA_LOADED( bi ) ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_db_open(): "
 			"test failed, schema map not loaded - exiting\n",
 			0, 0, 0 );
 		return 1;
 	}
+
+	/*
+	 * Prepare ID selection query
+	 */
+	if ( bi->sql_id_query == NULL ) {
+		/* no custom id_query provided */
+		if ( bi->sql_upper_func.bv_val == NULL ) {
+			backsql_strcat_x( &bb, NULL, backsql_id_query, "dn=?", NULL );
+
+		} else {
+			if ( BACKSQL_HAS_LDAPINFO_DN_RU( bi ) ) {
+				backsql_strcat_x( &bb, NULL, backsql_id_query,
+						"dn_ru=?", NULL );
+			} else {
+				if ( BACKSQL_USE_REVERSE_DN( bi ) ) {
+					backsql_strfcat_x( &bb, NULL, "sbl",
+							backsql_id_query,
+							&bi->sql_upper_func, 
+							(ber_len_t)STRLENOF( "(dn)=?" ), "(dn)=?" );
+				} else {
+					backsql_strfcat_x( &bb, NULL, "sblbcb",
+							backsql_id_query,
+							&bi->sql_upper_func, 
+							(ber_len_t)STRLENOF( "(dn)=" ), "(dn)=",
+							&bi->sql_upper_func_open, 
+							'?', 
+							&bi->sql_upper_func_close );
+				}
+			}
+		}
+		bi->sql_id_query = bb.bb_val.bv_val;
+	}
+
+	/*
+	 * Prepare children ID selection query
+	 */
+	BER_BVZERO( &bb.bb_val );
+	bb.bb_len = 0;
+	backsql_strfcat_x( &bb, NULL, "sbsb",
+			"SELECT COUNT(distinct subordinates.id) "
+			"FROM ldap_entries,ldap_entries ",
+			&bi->sql_aliasing, "subordinates "
+			"WHERE subordinates.parent=ldap_entries.id AND ",
+			&bi->sql_children_cond );
+	bi->sql_has_children_query = bb.bb_val.bv_val;
+ 
+	/*
+	 * Prepare DN and objectClass aliasing bit of query
+	 */
+	BER_BVZERO( &bb.bb_val );
+	bb.bb_len = 0;
+	backsql_strfcat_x( &bb, NULL, "sbbsbsbbsb",
+			" ", &bi->sql_aliasing, &bi->sql_aliasing_quote,
+			"objectClass", &bi->sql_aliasing_quote,
+			",ldap_entries.dn ", &bi->sql_aliasing,
+			&bi->sql_aliasing_quote, "dn", &bi->sql_aliasing_quote );
+	bi->sql_dn_oc_aliasing = bb.bb_val;
+ 
+	/* should never happen! */
+	assert( bd->be_nsuffix != NULL );
+	
+	if ( BER_BVISNULL( &bd->be_nsuffix[ 1 ] ) ) {
+		/* enable if only one suffix is defined */
+		bi->sql_flags |= BSQLF_USE_SUBTREE_SHORTCUT;
+	}
+
+	bi->sql_flags |= BSQLF_CHECK_SCHEMA;
 	
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_db_open(): "
 		"test succeeded, schema map loaded\n", 0, 0, 0 );
@@ -458,16 +567,22 @@ int
 backsql_db_close(
 	BackendDB	*bd )
 {
+	backsql_info 	*bi = (backsql_info*)bd->be_private;
+
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_db_close()\n", 0, 0, 0 );
+
+	backsql_conn_destroy( bi );
+
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_db_close()\n", 0, 0, 0 );
+
 	return 0;
 }
 
 int
 backsql_connection_destroy( Backend *bd, Connection *c )
 {
-	char		opbuf[ OPERATION_BUFFER_SIZE ];
-	Operation*	op = (Operation *)opbuf;
+	OperationBuffer opbuf;
+	Operation*	op = (Operation *) &opbuf;
 
 	op->o_hdr = (Opheader *)&op[ 1 ];
 	op->o_connid = c->c_connid;
