@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/extended.c,v 1.78.2.1 2005/01/20 18:03:54 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/extended.c,v 1.92 2006/03/06 23:48:14 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2005 The OpenLDAP Foundation.
+ * Copyright 1999-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,13 +39,8 @@
 #include "slap.h"
 #include "lber_pvt.h"
 
-#ifdef LDAP_SLAPI
-#include "slapi/slapi.h"
-#endif
-
 #define UNSUPPORTED_EXOP "unsupported extended operation"
 
-
 static struct extop_list {
 	struct extop_list *next;
 	struct berval oid;
@@ -60,22 +55,18 @@ static SLAP_EXTOP_MAIN_FN whoami_extop;
  * just a way to get built-in extops onto the extop list without
  * having a separate init routine for each built-in extop.
  */
-const struct berval slap_EXOP_CANCEL = BER_BVC(LDAP_EXOP_X_CANCEL);
-const struct berval slap_EXOP_WHOAMI = BER_BVC(LDAP_EXOP_X_WHO_AM_I);
-const struct berval slap_EXOP_MODIFY_PASSWD = BER_BVC(LDAP_EXOP_MODIFY_PASSWD);
-const struct berval slap_EXOP_START_TLS = BER_BVC(LDAP_EXOP_START_TLS);
-
 static struct {
 	const struct berval *oid;
 	slap_mask_t flags;
 	SLAP_EXTOP_MAIN_FN *ext_main;
 } builtin_extops[] = {
-	{ &slap_EXOP_CANCEL, SLAP_EXOP_HIDE, cancel_extop },
+#ifdef LDAP_X_TXN
+	{ &slap_EXOP_TXN_START, 0, txn_start_extop },
+	{ &slap_EXOP_TXN_END, 0, txn_end_extop },
+#endif
+	{ &slap_EXOP_CANCEL, 0, cancel_extop },
 	{ &slap_EXOP_WHOAMI, 0, whoami_extop },
 	{ &slap_EXOP_MODIFY_PASSWD, SLAP_EXOP_WRITES, passwd_extop },
-#ifdef HAVE_TLS
-	{ &slap_EXOP_START_TLS, 0, starttls_extop },
-#endif
 	{ NULL, 0, NULL }
 };
 
@@ -170,6 +161,8 @@ do_extended(
 
 	/* check for controls inappropriate for all extended operations */
 	if( get_manageDSAit( op ) == SLAP_CONTROL_CRITICAL ) {
+		Statslog( LDAP_DEBUG_STATS, "%s EXT oid=%s\n",
+		    op->o_log_prefix, op->ore_reqoid.bv_val, 0, 0, 0 );
 		send_ldap_error( op, rs,
 			LDAP_UNAVAILABLE_CRITICAL_EXTENSION,
 			"manageDSAit control inappropriate" );
@@ -194,26 +187,14 @@ fe_extended( Operation *op, SlapReply *rs )
 	struct extop_list	*ext = NULL;
 	struct berval		reqdata = BER_BVNULL;
 
-#if defined(LDAP_SLAPI) 
- 	Slapi_PBlock    	*pb = op->o_pb;
- 	SLAPI_FUNC      	funcAddr = NULL;
- 	int             	extop_rc;
- 	int             	msg_sent = FALSE;
-#endif /* defined(LDAP_SLAPI) */
-
 	if (op->ore_reqdata) {
 		reqdata = *op->ore_reqdata;
 	}
 
-#ifdef LDAP_SLAPI
-    /* NS-SLAPI extended operation */
-	slapi_int_get_extop_plugin( &op->ore_reqoid, &funcAddr );
-
-	if( !funcAddr && !(ext = find_extop(supp_ext_list, &op->ore_reqoid )))
-#else
-	if( !(ext = find_extop(supp_ext_list, &op->ore_reqoid )))
-#endif
-	{
+	ext = find_extop(supp_ext_list, &op->ore_reqoid );
+	if ( ext == NULL ) {
+		Statslog( LDAP_DEBUG_STATS, "%s EXT oid=%s\n",
+		    op->o_log_prefix, op->ore_reqoid.bv_val, 0, 0, 0 );
 		Debug( LDAP_DEBUG_ANY, "do_extended: unsupported operation \"%s\"\n",
 			op->ore_reqoid.bv_val, 0 ,0 );
 		send_ldap_error( op, rs, LDAP_PROTOCOL_ERROR,
@@ -226,69 +207,9 @@ fe_extended( Operation *op, SlapReply *rs )
 	Debug( LDAP_DEBUG_ARGS, "do_extended: oid=%s\n",
 		op->ore_reqoid.bv_val, 0 ,0 );
 
-#if defined(LDAP_SLAPI)
-	if ( funcAddr != NULL ) {
-		rs->sr_err = slapi_pblock_set( pb, SLAPI_EXT_OP_REQ_OID,
-				(void *)op->ore_reqoid.bv_val);
-		if ( rs->sr_err != LDAP_SUCCESS ) {
-			rs->sr_err = LDAP_OTHER;
-			goto done;
-		}
-
-		rs->sr_err = slapi_pblock_set( pb, SLAPI_EXT_OP_REQ_VALUE,
-				(void *)&reqdata);
-		if ( rs->sr_err != LDAP_SUCCESS ) {
-			rs->sr_err = LDAP_OTHER;
-			goto done;
-		}
-
-		rs->sr_err = slapi_int_pblock_set_operation( pb, op );
-		if ( rs->sr_err != LDAP_SUCCESS ) {
-			rs->sr_err = LDAP_OTHER;
-			goto done;
-		}
-
-		extop_rc = (*funcAddr)( pb );
-		if ( extop_rc == SLAPI_PLUGIN_EXTENDED_SENT_RESULT ) {
-			msg_sent = TRUE;
-
-		} else if ( extop_rc == SLAPI_PLUGIN_EXTENDED_NOT_HANDLED ) {
-			rs->sr_err = LDAP_PROTOCOL_ERROR;
-			rs->sr_text = UNSUPPORTED_EXOP;
-
-		} else {
-			rs->sr_err = slapi_pblock_get( pb, SLAPI_EXT_OP_RET_OID,
-					&rs->sr_rspoid);
-			if ( rs->sr_err != LDAP_SUCCESS ) {
-				goto done2;
-			}
-
-			rs->sr_err = slapi_pblock_get( pb, SLAPI_EXT_OP_RET_VALUE,
-					&rs->sr_rspdata);
-			if ( rs->sr_err != LDAP_SUCCESS ) {
-				goto done2;
-			}
-
-			rs->sr_err = extop_rc;
-			send_ldap_extended( op, rs );
-			msg_sent = TRUE;
-		}
-
-done2:;
-		if ( rs->sr_err != LDAP_SUCCESS && msg_sent == FALSE ) {
-			send_ldap_result( op, rs );
-		}
-
-		if ( rs->sr_rspoid != NULL ) {
-			ch_free( (char *)rs->sr_rspoid );
-		}
-
-		if ( rs->sr_rspdata != NULL ) {
-			ber_bvfree( rs->sr_rspdata );
-		}
-	} else
-#endif /* defined( LDAP_SLAPI ) */
 	{ /* start of OpenLDAP extended operation */
+		BackendDB	*bd = op->o_bd;
+
 		rs->sr_err = (ext->ext_main)( op, rs );
 
 		if( rs->sr_err != SLAPD_ABANDON ) {
@@ -302,6 +223,8 @@ done2:;
 				}
 			}
 
+			if ( op->o_bd == NULL )
+				op->o_bd = bd;
 			send_ldap_extended( op, rs );
 
 			if ( rs->sr_ref != default_referral ) {
@@ -324,32 +247,63 @@ done:;
 }
 
 int
-load_extop(
-	struct berval *ext_oid,
+load_extop2(
+	const struct berval *ext_oid,
 	slap_mask_t ext_flags,
-	SLAP_EXTOP_MAIN_FN *ext_main )
+	SLAP_EXTOP_MAIN_FN *ext_main,
+	unsigned flags )
 {
-	struct extop_list *ext;
+	struct berval		oidm = BER_BVNULL;
+	struct extop_list	*ext;
+	int			insertme = 0;
 
-	if( ext_oid == NULL || ext_oid->bv_val == NULL ||
-		ext_oid->bv_val[0] == '\0' || ext_oid->bv_len == 0 ) return -1; 
-	if(!ext_main) return -1; 
+	if ( !ext_main ) {
+		return -1; 
+	}
 
-	ext = ch_calloc(1, sizeof(struct extop_list) + ext_oid->bv_len + 1);
-	if (ext == NULL)
-		return(-1);
+	if ( ext_oid == NULL || BER_BVISNULL( ext_oid ) || BER_BVISEMPTY( ext_oid ) ) {
+		return -1; 
+	}
+
+	if ( numericoidValidate( NULL, (struct berval *)ext_oid ) != LDAP_SUCCESS ) {
+		oidm.bv_val = oidm_find( ext_oid->bv_val );
+		if ( ext_oid == NULL ) {
+			return -1;
+		}
+		oidm.bv_len = strlen( oidm.bv_val );
+		ext_oid = &oidm;
+	}
+
+	for ( ext = supp_ext_list; ext; ext = ext->next ) {
+		if ( bvmatch( ext_oid, &ext->oid ) ) {
+			if ( flags == 1 ) {
+				break;
+			}
+			return -1;
+		}
+	}
+
+	if ( flags == 0 || ext == NULL ) {
+		ext = ch_calloc( 1, sizeof(struct extop_list) + ext_oid->bv_len + 1 );
+		if ( ext == NULL ) {
+			return(-1);
+		}
+
+		ext->oid.bv_val = (char *)(ext + 1);
+		AC_MEMCPY( ext->oid.bv_val, ext_oid->bv_val, ext_oid->bv_len );
+		ext->oid.bv_len = ext_oid->bv_len;
+		ext->oid.bv_val[ext->oid.bv_len] = '\0';
+
+		insertme = 1;
+	}
 
 	ext->flags = ext_flags;
-
-	ext->oid.bv_val = (char *)(ext + 1);
-	AC_MEMCPY( ext->oid.bv_val, ext_oid->bv_val, ext_oid->bv_len );
-	ext->oid.bv_len = ext_oid->bv_len;
-	ext->oid.bv_val[ext->oid.bv_len] = '\0';
-
 	ext->ext_main = ext_main;
-	ext->next = supp_ext_list;
 
-	supp_ext_list = ext;
+	if ( insertme ) {
+		ext->next = supp_ext_list;
+		supp_ext_list = ext;
+	}
 
 	return(0);
 }
@@ -359,10 +313,10 @@ extops_init (void)
 {
 	int i;
 
-	for (i = 0; builtin_extops[i].oid != NULL; i++) {
-		load_extop((struct berval *)builtin_extops[i].oid,
+	for ( i = 0; builtin_extops[i].oid != NULL; i++ ) {
+		load_extop( (struct berval *)builtin_extops[i].oid,
 			builtin_extops[i].flags,
-			builtin_extops[i].ext_main);
+			builtin_extops[i].ext_main );
 	}
 	return(0);
 }
@@ -393,6 +347,8 @@ find_extop( struct extop_list *list, struct berval *oid )
 }
 
 
+const struct berval slap_EXOP_WHOAMI = BER_BVC(LDAP_EXOP_WHO_AM_I);
+
 static int
 whoami_extop (
 	Operation *op,
@@ -406,9 +362,13 @@ whoami_extop (
 		return LDAP_PROTOCOL_ERROR;
 	}
 
+	Statslog( LDAP_DEBUG_STATS, "%s WHOAMI\n",
+	    op->o_log_prefix, 0, 0, 0, 0 );
+
 	op->o_bd = op->o_conn->c_authz_backend;
 	if( backend_check_restrictions( op, rs,
-		(struct berval *)&slap_EXOP_WHOAMI ) != LDAP_SUCCESS ) {
+		(struct berval *)&slap_EXOP_WHOAMI ) != LDAP_SUCCESS )
+	{
 		return rs->sr_err;
 	}
 
