@@ -1,8 +1,8 @@
 /* limits.c - routines to handle regex-based size and time limits */
-/* $OpenLDAP: pkg/ldap/servers/slapd/limits.c,v 1.62.2.1 2005/01/20 18:03:55 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/limits.c,v 1.73.2.1 2006/04/11 16:07:17 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -22,6 +22,7 @@
 #include <ac/string.h>
 
 #include "slap.h"
+#include "lutil.h"
 
 /* define to get an error if requesting limit higher than hard */
 #undef ABOVE_HARD_LIMIT_IS_ERROR
@@ -71,8 +72,8 @@ limits_get(
 {
 	struct slap_limits **lm;
 
-	assert( op );
-	assert( limit );
+	assert( op != NULL );
+	assert( limit != NULL );
 
 	Debug( LDAP_DEBUG_TRACE, "==> limits_get: %s dn=\"%s\"\n",
 			op->o_log_prefix,
@@ -92,7 +93,7 @@ limits_get(
 
 		switch ( style ) {
 		case SLAP_LIMITS_EXACT:
-			if ( ndn->bv_len == 0 ) {
+			if ( BER_BVISEMPTY( ndn ) ) {
 				break;
 			}
 
@@ -129,7 +130,7 @@ limits_get(
 		case SLAP_LIMITS_CHILDREN: {
 			size_t d;
 			
-			if ( ndn->bv_len == 0 ) {
+			if ( BER_BVISEMPTY( ndn ) ) {
 				break;
 			}
 
@@ -146,7 +147,7 @@ limits_get(
 				}
 			} else {
 				/* check for unescaped rdn separator */
-				if ( !DN_SEPARATOR( ndn->bv_val[d-1] ) ) {
+				if ( !DN_SEPARATOR( ndn->bv_val[d - 1] ) ) {
 					break;
 				}
 			}
@@ -179,7 +180,7 @@ limits_get(
 		}
 
 		case SLAP_LIMITS_REGEX:
-			if ( ndn->bv_len == 0 ) {
+			if ( BER_BVISEMPTY( ndn ) ) {
 				break;
 			}
 			if ( regexec( &lm[0]->lm_regex, ndn->bv_val,
@@ -193,7 +194,7 @@ limits_get(
 			break;
 
 		case SLAP_LIMITS_ANONYMOUS:
-			if ( ndn->bv_len == 0 ) {
+			if ( BER_BVISEMPTY( ndn ) ) {
 				Debug( LDAP_DEBUG_TRACE, "<== limits_get: type=DN match=%s\n",
 						limits2str( style ), 0, 0 );
 				*limit = &lm[0]->lm_limits;
@@ -202,7 +203,7 @@ limits_get(
 			break;
 
 		case SLAP_LIMITS_USERS:
-			if ( ndn->bv_len != 0 ) {
+			if ( !BER_BVISEMPTY( ndn ) ) {
 				*limit = &lm[0]->lm_limits;
 				Debug( LDAP_DEBUG_TRACE, "<== limits_get: type=DN match=%s\n",
 						limits2str( style ), 0, 0 );
@@ -237,8 +238,8 @@ limits_add(
 	struct slap_limits	*lm;
 	unsigned		type, style;
 	
-	assert( be );
-	assert( limit );
+	assert( be != NULL );
+	assert( limit != NULL );
 
 	type = flags & SLAP_LIMITS_TYPE_MASK;
 	style = flags & SLAP_LIMITS_MASK;
@@ -270,8 +271,8 @@ limits_add(
 		{
 			int rc;
 			struct berval bv;
-			bv.bv_val = (char *) pattern;
-			bv.bv_len = strlen( pattern );
+
+			ber_str2bv( pattern, 0, 0, &bv );
 
 			rc = dnNormalize( 0, NULL, NULL, &bv, &lm->lm_pat, NULL );
 			if ( rc != LDAP_SUCCESS ) {
@@ -296,15 +297,14 @@ limits_add(
 	case SLAP_LIMITS_USERS:
 	case SLAP_LIMITS_ANY:
 		lm->lm_flags = style | type;
-		lm->lm_pat.bv_val = NULL;
-		lm->lm_pat.bv_len = 0;
+		BER_BVZERO( &lm->lm_pat );
 		break;
 	}
 
 	switch ( type ) {
 	case SLAP_LIMITS_TYPE_GROUP:
-		assert( group_oc );
-		assert( group_ad );
+		assert( group_oc != NULL );
+		assert( group_ad != NULL );
 		lm->lm_group_oc = group_oc;
 		lm->lm_group_ad = group_ad;
 		break;
@@ -341,7 +341,7 @@ limits_parse(
 	ObjectClass		*group_oc = NULL;
 	AttributeDescription	*group_ad = NULL;
 
-	assert( be );
+	assert( be != NULL );
 
 	if ( argc < 3 ) {
 		Debug( LDAP_DEBUG_ANY,
@@ -636,8 +636,8 @@ limits_parse_one(
 	struct slap_limits_set 	*limit
 )
 {
-	assert( arg );
-	assert( limit );
+	assert( arg != NULL );
+	assert( limit != NULL );
 
 	if ( strncasecmp( arg, "time", STRLENOF( "time" ) ) == 0 ) {
 		arg += STRLENOF( "time" );
@@ -650,14 +650,9 @@ limits_parse_one(
 					limit->lms_t_soft = -1;
 
 				} else {
-					char	*next = NULL;
-					int	soft = strtol( arg, &next, 10 );
+					int	soft;
 
-					if ( next == arg || next[ 0 ] != '\0' ) {
-						return( 1 );
-					}
-
-					if ( soft < -1 ) {
+					if ( lutil_atoi( &soft, arg ) != 0 || soft < -1 ) {
 						return( 1 );
 					}
 
@@ -677,14 +672,9 @@ limits_parse_one(
 					limit->lms_t_hard = -1;
 
 				} else {
-					char	*next = NULL;
-					int	hard = strtol( arg, &next, 10 );
+					int	hard;
 
-					if ( next == arg || next[ 0 ] != '\0' ) {
-						return( 1 );
-					}
-
-					if ( hard < -1 ) {
+					if ( lutil_atoi( &hard, arg ) != 0 || hard < -1 ) {
 						return( 1 );
 					}
 
@@ -709,10 +699,9 @@ limits_parse_one(
 				limit->lms_t_soft = -1;
 
 			} else {
-				char	*next = NULL;
-
-				limit->lms_t_soft = strtol( arg, &next, 10 );
-				if ( next == arg || limit->lms_t_soft < -1 ) {
+				if ( lutil_atoi( &limit->lms_t_soft, arg ) != 0 
+					|| limit->lms_t_soft < -1 )
+				{
 					return( 1 );
 				}
 			}
@@ -733,14 +722,9 @@ limits_parse_one(
 					limit->lms_s_soft = -1;
 
 				} else {
-					char	*next = NULL;
-					int	soft = strtol( arg, &next, 10 );
+					int	soft;
 
-					if ( next == arg || next[ 0 ] != '\0' ) {
-						return( 1 );
-					}
-
-					if ( soft < -1 ) {
+					if ( lutil_atoi( &soft, arg ) != 0 || soft < -1 ) {
 						return( 1 );
 					}
 
@@ -760,14 +744,9 @@ limits_parse_one(
 					limit->lms_s_hard = -1;
 
 				} else {
-					char	*next = NULL;
-					int	hard = strtol( arg, &next, 10 );
+					int	hard;
 
-					if ( next == arg || next[ 0 ] != '\0' ) {
-						return( 1 );
-					}
-
-					if ( hard < -1 ) {
+					if ( lutil_atoi( &hard, arg ) != 0 || hard < -1 ) {
 						return( 1 );
 					}
 
@@ -791,14 +770,9 @@ limits_parse_one(
 					limit->lms_s_unchecked = 0;
 
 				} else {
-					char	*next = NULL;
-					int	unchecked = strtol( arg, &next, 10 );
+					int	unchecked;
 
-					if ( next == arg || next[ 0 ] != '\0' ) {
-						return( 1 );
-					}
-
-					if ( unchecked < -1 ) {
+					if ( lutil_atoi( &unchecked, arg ) != 0 || unchecked < -1 ) {
 						return( 1 );
 					}
 
@@ -818,14 +792,9 @@ limits_parse_one(
 					limit->lms_s_pr = -1;
 
 				} else {
-					char	*next = NULL;
-					int	pr = strtol( arg, &next, 10 );
+					int	pr;
 
-					if ( next == arg || next[ 0 ] != '\0' ) {
-						return( 1 );
-					}
-
-					if ( pr < -1 ) {
+					if ( lutil_atoi( &pr, arg ) != 0 || pr < -1 ) {
 						return( 1 );
 					}
 
@@ -849,15 +818,9 @@ limits_parse_one(
 					limit->lms_s_pr_total = 0;
 
 				} else {
-					char	*next = NULL;
 					int	total;
 
-					total = strtol( arg, &next, 10 );
-					if ( next == arg || next[ 0 ] != '\0' ) {
-						return( 1 );
-					}
-
-					if ( total < -1 ) {
+					if ( lutil_atoi( &total, arg ) != 0 || total < -1 ) {
 						return( 1 );
 					}
 
@@ -882,10 +845,9 @@ limits_parse_one(
 				limit->lms_s_soft = -1;
 
 			} else {
-				char	*next = NULL;
-
-				limit->lms_s_soft = strtol( arg, &next, 10 );
-				if ( next == arg || limit->lms_s_soft < -1 ) {
+				if ( lutil_atoi( &limit->lms_s_soft, arg ) != 0
+					|| limit->lms_s_soft < -1 )
+				{
 					return( 1 );
 				}
 			}
@@ -899,12 +861,237 @@ limits_parse_one(
 	return 0;
 }
 
+static const char *lmpats[] = {
+	"base",
+	"base",
+	"onelevel",
+	"subtree",
+	"children",
+	"regex",
+	"anonymous",
+	"users",
+	"*"
+};
+
+#define WHATSLEFT	( buflen - ( ptr - bv->bv_val ) )
+
+/* Caller must provide an adequately sized buffer in bv */
+int
+limits_unparse( struct slap_limits *lim, struct berval *bv, ber_len_t buflen )
+{
+	struct berval btmp;
+	char *ptr;
+	int lm;
+
+	if ( !bv || !bv->bv_val ) return -1;
+
+	ptr = bv->bv_val;
+
+	if (( lim->lm_flags & SLAP_LIMITS_TYPE_MASK ) == SLAP_LIMITS_TYPE_GROUP ) {
+		if ( WHATSLEFT <= STRLENOF( "group/" "/" "=\"" "\"" )
+				+ lim->lm_group_oc->soc_cname.bv_len
+				+ lim->lm_group_ad->ad_cname.bv_len
+				+ lim->lm_pat.bv_len ) return -1;
+
+		ptr = lutil_strcopy( ptr, "group/" );
+		ptr = lutil_strcopy( ptr, lim->lm_group_oc->soc_cname.bv_val );
+		*ptr++ = '/';
+		ptr = lutil_strcopy( ptr, lim->lm_group_ad->ad_cname.bv_val );
+		ptr = lutil_strcopy( ptr, "=\"" );
+		ptr = lutil_strcopy( ptr, lim->lm_pat.bv_val );
+		*ptr++ = '"';
+	} else {
+		lm = lim->lm_flags & SLAP_LIMITS_MASK;
+		switch( lm ) {
+		case SLAP_LIMITS_ANONYMOUS:
+		case SLAP_LIMITS_USERS:
+		case SLAP_LIMITS_ANY:
+			if ( WHATSLEFT <= strlen( lmpats[lm] ) ) return -1;
+			ptr = lutil_strcopy( ptr, lmpats[lm] );
+			break;
+		case SLAP_LIMITS_UNDEFINED:
+		case SLAP_LIMITS_EXACT:
+		case SLAP_LIMITS_ONE:
+		case SLAP_LIMITS_SUBTREE:
+		case SLAP_LIMITS_CHILDREN:
+		case SLAP_LIMITS_REGEX:
+			if ( WHATSLEFT <= STRLENOF( "dn." "=" "\"" "\"" )
+					+ strlen( lmpats[lm] ) + lim->lm_pat.bv_len ) return -1;
+			ptr = lutil_strcopy( ptr, "dn." );
+			ptr = lutil_strcopy( ptr, lmpats[lm] );
+			*ptr++ = '=';
+			*ptr++ = '"';
+			ptr = lutil_strcopy( ptr, lim->lm_pat.bv_val );
+			*ptr++ = '"';
+			break;
+		}
+	}
+	if ( WHATSLEFT <= STRLENOF( " " ) ) return -1;
+	*ptr++ = ' ';
+	bv->bv_len = ptr - bv->bv_val;
+	btmp.bv_val = ptr;
+	btmp.bv_len = 0;
+	if ( limits_unparse_one( &lim->lm_limits,
+			SLAP_LIMIT_SIZE|SLAP_LIMIT_TIME,
+			&btmp, WHATSLEFT ) )
+	{
+		return -1;
+	}
+	bv->bv_len += btmp.bv_len;
+	return 0;
+}
+
+/* Caller must provide an adequately sized buffer in bv */
+int
+limits_unparse_one( struct slap_limits_set *lim, int which, struct berval *bv, ber_len_t buflen )
+{
+	char *ptr;
+
+	if ( !bv || !bv->bv_val ) return;
+
+	ptr = bv->bv_val;
+
+	if ( which & SLAP_LIMIT_SIZE ) {
+		if ( lim->lms_s_soft != SLAPD_DEFAULT_SIZELIMIT ) {
+
+			/* If same as global limit, drop it */
+			if ( lim != &frontendDB->be_def_limit &&
+				lim->lms_s_soft == frontendDB->be_def_limit.lms_s_soft )
+			{
+				goto s_hard;
+			/* If there's also a hard limit, fully qualify this one */
+			} else if ( lim->lms_s_hard ) {
+				if ( WHATSLEFT <= STRLENOF( " size.soft=" ) ) return -1;
+				ptr = lutil_strcopy( ptr, " size.soft=" );
+
+			/* If doing both size & time, qualify this */
+			} else if ( which & SLAP_LIMIT_TIME ) {
+				if ( WHATSLEFT <= STRLENOF( " size=" ) ) return -1;
+				ptr = lutil_strcopy( ptr, " size=" );
+			}
+
+			if ( lim->lms_s_soft == -1 ) {
+				if ( WHATSLEFT <= STRLENOF( "unlimited" ) ) return -1;
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			} else {
+				ptr += snprintf( ptr, WHATSLEFT, "%d", lim->lms_s_soft );
+				if ( WHATSLEFT < 0 ) return -1;
+			}
+			*ptr++ = ' ';
+		}
+s_hard:
+		if ( lim->lms_s_hard ) {
+			if ( WHATSLEFT <= STRLENOF( " size.hard=" ) ) return -1;
+			ptr = lutil_strcopy( ptr, " size.hard=" );
+			if ( lim->lms_s_hard == -1 ) {
+				if ( WHATSLEFT <= STRLENOF( "unlimited" ) ) return -1;
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			} else {
+				ptr += snprintf( ptr, WHATSLEFT, "%d", lim->lms_s_hard );
+				if ( WHATSLEFT < 0 ) return -1;
+			}
+			*ptr++ = ' ';
+		}
+		if ( lim->lms_s_unchecked != -1 ) {
+			if ( WHATSLEFT <= STRLENOF( " size.unchecked=" ) ) return -1;
+			ptr = lutil_strcopy( ptr, " size.unchecked=" );
+			if ( lim->lms_s_unchecked == 0 ) {
+				if ( WHATSLEFT <= STRLENOF( "disabled" ) ) return -1;
+				ptr = lutil_strcopy( ptr, "disabled" );
+			} else {
+				ptr += snprintf( ptr, WHATSLEFT, "%d", lim->lms_s_unchecked );
+				if ( WHATSLEFT < 0 ) return -1;
+			}
+			*ptr++ = ' ';
+		}
+		if ( lim->lms_s_pr_hide ) {
+			if ( WHATSLEFT <= STRLENOF( " size.pr=noEstimate " ) ) return -1;
+			ptr = lutil_strcopy( ptr, " size.pr=noEstimate " );
+		}
+		if ( lim->lms_s_pr ) {
+			if ( WHATSLEFT <= STRLENOF( " size.pr=" ) ) return -1;
+			ptr = lutil_strcopy( ptr, " size.pr=" );
+			if ( lim->lms_s_pr == -1 ) {
+				if ( WHATSLEFT <= STRLENOF( "unlimited" ) ) return -1;
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			} else {
+				ptr += snprintf( ptr, WHATSLEFT, "%d", lim->lms_s_pr );
+				if ( WHATSLEFT < 0 ) return -1;
+			}
+			*ptr++ = ' ';
+		}
+		if ( lim->lms_s_pr_total ) {
+			if ( WHATSLEFT <= STRLENOF( " size.prtotal=" ) ) return -1;
+			ptr = lutil_strcopy( ptr, " size.prtotal=" );
+			if ( lim->lms_s_pr_total == -1 ) {
+				if ( WHATSLEFT <= STRLENOF( "unlimited" ) ) return -1;
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			} else if ( lim->lms_s_pr_total == -2 ) {
+				if ( WHATSLEFT <= STRLENOF( "disabled" ) ) return -1;
+				ptr = lutil_strcopy( ptr, "disabled" );
+			} else {
+				ptr += snprintf( ptr, WHATSLEFT, "%d", lim->lms_s_pr_total );
+				if ( WHATSLEFT < 0 ) return -1;
+			}
+			*ptr++ = ' ';
+		}
+	}
+
+	if ( which & SLAP_LIMIT_TIME ) {
+		if ( lim->lms_t_soft != SLAPD_DEFAULT_TIMELIMIT ) {
+
+			/* If same as global limit, drop it */
+			if ( lim != &frontendDB->be_def_limit &&
+				lim->lms_t_soft == frontendDB->be_def_limit.lms_t_soft )
+			{
+				goto t_hard;
+
+			/* If there's also a hard limit, fully qualify this one */
+			} else if ( lim->lms_t_hard ) {
+				if ( WHATSLEFT <= STRLENOF( " time.soft=" ) ) return -1;
+				ptr = lutil_strcopy( ptr, " time.soft=" );
+
+			/* If doing both size & time, qualify this */
+			} else if ( which & SLAP_LIMIT_SIZE ) {
+				if ( WHATSLEFT <= STRLENOF( " time=" ) ) return -1;
+				ptr = lutil_strcopy( ptr, " time=" );
+			}
+
+			if ( lim->lms_t_soft == -1 ) {
+				if ( WHATSLEFT <= STRLENOF( "unlimited" ) ) return -1;
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			} else {
+				ptr += snprintf( ptr, WHATSLEFT, "%d", lim->lms_t_soft );
+				if ( WHATSLEFT < 0 ) return -1;
+			}
+			*ptr++ = ' ';
+		}
+t_hard:
+		if ( lim->lms_t_hard ) {
+			if ( WHATSLEFT <= STRLENOF( " time.hard=" ) ) return -1;
+			ptr = lutil_strcopy( ptr, " time.hard=" );
+			if ( lim->lms_t_hard == -1 ) {
+				if ( WHATSLEFT <= STRLENOF( "unlimited" ) ) return -1;
+				ptr = lutil_strcopy( ptr, "unlimited" );
+			} else {
+				ptr += snprintf( ptr, WHATSLEFT, "%d", lim->lms_t_hard );
+				if ( WHATSLEFT < 0 ) return -1;
+			}
+			*ptr++ = ' ';
+		}
+	}
+	if ( ptr != bv->bv_val ) {
+		ptr--;
+		*ptr = '\0';
+		bv->bv_len = ptr - bv->bv_val;
+	}
+}
 
 int
 limits_check( Operation *op, SlapReply *rs )
 {
-	assert( op );
-	assert( rs );
+	assert( op != NULL );
+	assert( rs != NULL );
 	/* FIXME: should this be always true? */
 	assert( op->o_tag == LDAP_REQ_SEARCH);
 
@@ -1100,6 +1287,9 @@ limits_check( Operation *op, SlapReply *rs )
 						return -1;
 					}
 					op->ors_slimit = slimit;
+
+				} else if ( slimit == 0 ) {
+					op->ors_slimit = 0;
 				}
 
 			} else {
@@ -1147,3 +1337,37 @@ limits_check( Operation *op, SlapReply *rs )
 	return 0;
 }
 
+void
+limits_destroy( 
+	struct slap_limits	**lm )
+{
+	int		i;
+
+	if ( lm == NULL ) {
+		return;
+	}
+
+	for ( i = 0; lm[ i ]; i++ ) {
+		switch ( lm[ i ]->lm_flags & SLAP_LIMITS_MASK ) {
+		case SLAP_LIMITS_REGEX:
+			regfree( &lm[ i ]->lm_regex );
+			break;
+
+		case SLAP_LIMITS_EXACT:
+		case SLAP_LIMITS_ONE:
+		case SLAP_LIMITS_SUBTREE:
+		case SLAP_LIMITS_CHILDREN:
+			if ( !BER_BVISNULL( &lm[ i ]->lm_pat ) ) {
+				ch_free( lm[ i ]->lm_pat.bv_val );
+			}
+			break;
+
+		default:
+			break;
+		}
+
+		ch_free( lm[ i ] );
+	}
+
+	ch_free( lm );
+}
