@@ -1,8 +1,8 @@
 /* rwmmap.c - rewrite/mapping routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwmmap.c,v 1.14.2.3 2005/01/24 21:28:53 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwmmap.c,v 1.31.2.1 2006/04/18 17:33:12 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2005 The OpenLDAP Foundation.
+ * Copyright 1999-2006 The OpenLDAP Foundation.
  * Portions Copyright 1999-2003 Howard Chu.
  * Portions Copyright 2000-2003 Pierangelo Masarati.
  * All rights reserved.
@@ -71,7 +71,7 @@ rwm_map_init( struct ldapmap *lm, struct ldapmapping **m )
 	const char		*text;
 	int			rc;
 
-	assert( m );
+	assert( m != NULL );
 
 	*m = NULL;
 	
@@ -82,19 +82,22 @@ rwm_map_init( struct ldapmap *lm, struct ldapmapping **m )
 	}
 
 	/* FIXME: I don't think this is needed any more... */
-	rc = slap_str2ad( "objectClass", &mapping->m_src_ad, &text );
+	rc = slap_str2ad( "objectClass", &mapping[0].m_src_ad, &text );
 	if ( rc != LDAP_SUCCESS ) {
+		ch_free( mapping );
 		return rc;
 	}
 
-	mapping->m_dst_ad = mapping->m_src_ad;
-	ber_dupbv( &mapping->m_dst, &mapping->m_src_ad->ad_cname );
-	ber_dupbv( &mapping->m_dst, &mapping->m_src );
+	mapping[0].m_dst_ad = mapping[0].m_src_ad;
+	ber_dupbv( &mapping[0].m_src, &mapping[0].m_src_ad->ad_cname );
+	ber_dupbv( &mapping[0].m_dst, &mapping[0].m_src );
 
-	mapping[1].m_src = mapping->m_src;
-	mapping[1].m_dst = mapping->m_dst;
+	mapping[1].m_src = mapping[0].m_src;
+	mapping[1].m_dst = mapping[0].m_dst;
+	mapping[1].m_src_ad = mapping[0].m_src_ad;
+	mapping[1].m_dst_ad = mapping[1].m_src_ad;
 
-	avl_insert( &lm->map, (caddr_t)mapping, 
+	avl_insert( &lm->map, (caddr_t)&mapping[0], 
 			rwm_mapping_cmp, rwm_mapping_dup );
 	avl_insert( &lm->remap, (caddr_t)&mapping[1], 
 			rwm_mapping_cmp, rwm_mapping_dup );
@@ -110,7 +113,11 @@ rwm_mapping( struct ldapmap *map, struct berval *s, struct ldapmapping **m, int
 	Avlnode *tree;
 	struct ldapmapping fmapping;
 
-	assert( m );
+	if ( map == NULL ) {
+		return 0;
+	}
+
+	assert( m != NULL );
 
 	if ( remap == RWM_REMAP ) {
 		tree = map->remap;
@@ -154,16 +161,15 @@ rwm_map( struct ldapmap *map, struct berval *s, struct berval *bv, int remap )
  */
 int
 rwm_map_attrnames(
-		struct ldapmap *at_map,
-		struct ldapmap *oc_map,
-		AttributeName *an,
-		AttributeName **anp,
-		int remap
-)
+	struct ldapmap	*at_map,
+	struct ldapmap	*oc_map,
+	AttributeName	*an,
+	AttributeName	**anp,
+	int		remap )
 {
 	int		i, j;
 
-	assert( anp );
+	assert( anp != NULL );
 
 	*anp = NULL;
 
@@ -244,7 +250,6 @@ rwm_map_attrnames(
 			at_drop_missing = rwm_mapping( at_map, &an[i].an_name, &m, remap );
 		
 			if ( at_drop_missing || !m ) {
-
 				oc_drop_missing = rwm_mapping( oc_map, &an[i].an_name, &m, remap );
 
 				/* if both at_map and oc_map required to drop missing,
@@ -299,6 +304,7 @@ rwm_map_attrnames(
 	if ( j == 0 && i != 0 ) {
 		memset( &(*anp)[0], 0, sizeof( AttributeName ) );
 		BER_BVSTR( &(*anp)[0].an_name, LDAP_NO_ATTRS );
+		j = 1;
 	}
 	memset( &(*anp)[j], 0, sizeof( AttributeName ) );
 
@@ -307,11 +313,10 @@ rwm_map_attrnames(
 
 int
 rwm_map_attrs(
-		struct ldapmap *at_map,
-		AttributeName *an,
-		int remap,
-		char ***mapped_attrs
-)
+	struct ldapmap	*at_map,
+	AttributeName	*an,
+	int		remap,
+	char		***mapped_attrs )
 {
 	int i, j;
 	char **na;
@@ -321,9 +326,8 @@ rwm_map_attrs(
 		return LDAP_SUCCESS;
 	}
 
-	for ( i = 0; !BER_BVISNULL( &an[ i ].an_name ); i++ ) {
-		/*  */
-	}
+	for ( i = 0; !BER_BVISNULL( &an[ i ].an_name ); i++ )
+		/* count'em */ ;
 
 	na = (char **)ch_calloc( i + 1, sizeof( char * ) );
 	if ( na == NULL ) {
@@ -359,12 +363,12 @@ rwm_map_attrs(
 
 static int
 map_attr_value(
-		dncookie		*dc,
-		AttributeDescription 	**adp,
-		struct berval		*mapped_attr,
-		struct berval		*value,
-		struct berval		*mapped_value,
-		int			remap )
+	dncookie		*dc,
+	AttributeDescription 	**adp,
+	struct berval		*mapped_attr,
+	struct berval		*value,
+	struct berval		*mapped_value,
+	int			remap )
 {
 	struct berval		vtmp = BER_BVNULL;
 	int			freeval = 0;
@@ -386,7 +390,8 @@ map_attr_value(
 	if ( value != NULL ) {
 		assert( mapped_value != NULL );
 
-		if ( ad->ad_type->sat_syntax == slap_schema.si_syn_distinguishedName )
+		if ( ad->ad_type->sat_syntax == slap_schema.si_syn_distinguishedName
+				|| ( mapping != NULL && mapping->m_dst_ad->ad_type->sat_syntax == slap_schema.si_syn_distinguishedName ) )
 		{
 			dncookie 	fdc = *dc;
 			int		rc;
@@ -439,27 +444,38 @@ map_attr_value(
 
 static int
 rwm_int_filter_map_rewrite(
-		dncookie		*dc,
-		Filter			*f,
-		struct berval		*fstr )
+	Operation		*op,
+	dncookie		*dc,
+	Filter			*f,
+	struct berval		*fstr )
 {
 	int		i;
 	Filter		*p;
 	struct berval	atmp,
 			vtmp,
-			tmp;
+			*tmp;
 	static struct berval
-			ber_bvfalse = BER_BVC( "(?=false)" ),
-			ber_bvtrue = BER_BVC( "(?=true)" ),
+			/* better than nothing... */
+			ber_bvfalse = BER_BVC( "(!(objectClass=*))" ),
+			ber_bvtf_false = BER_BVC( "(|)" ),
+			/* better than nothing... */
+			ber_bvtrue = BER_BVC( "(objectClass=*)" ),
+			ber_bvtf_true = BER_BVC( "(&)" ),
+#if 0
+			/* no longer needed; preserved for completeness */
 			ber_bvundefined = BER_BVC( "(?=undefined)" ),
+#endif
 			ber_bverror = BER_BVC( "(?=error)" ),
 			ber_bvunknown = BER_BVC( "(?=unknown)" ),
 			ber_bvnone = BER_BVC( "(?=none)" );
 	ber_len_t	len;
 
+	assert( fstr != NULL );
+	BER_BVZERO( fstr );
+
 	if ( f == NULL ) {
 		ber_dupbv( fstr, &ber_bvnone );
-		return -1;
+		return LDAP_OTHER;
 	}
 
 	switch ( f->f_choice ) {
@@ -467,7 +483,7 @@ rwm_int_filter_map_rewrite(
 		if ( map_attr_value( dc, &f->f_av_desc, &atmp,
 					&f->f_av_value, &vtmp, RWM_MAP ) )
 		{
-			return -1;
+			goto computed;
 		}
 
 		fstr->bv_len = atmp.bv_len + vtmp.bv_len + STRLENOF( "(=)" );
@@ -483,7 +499,7 @@ rwm_int_filter_map_rewrite(
 		if ( map_attr_value( dc, &f->f_av_desc, &atmp,
 					&f->f_av_value, &vtmp, RWM_MAP ) )
 		{
-			return -1;
+			goto computed;
 		}
 
 		fstr->bv_len = atmp.bv_len + vtmp.bv_len + STRLENOF( "(>=)" );
@@ -499,7 +515,7 @@ rwm_int_filter_map_rewrite(
 		if ( map_attr_value( dc, &f->f_av_desc, &atmp,
 					&f->f_av_value, &vtmp, RWM_MAP ) )
 		{
-			return -1;
+			goto computed;
 		}
 
 		fstr->bv_len = atmp.bv_len + vtmp.bv_len + STRLENOF( "(<=)" );
@@ -515,7 +531,7 @@ rwm_int_filter_map_rewrite(
 		if ( map_attr_value( dc, &f->f_av_desc, &atmp,
 					&f->f_av_value, &vtmp, RWM_MAP ) )
 		{
-			return -1;
+			goto computed;
 		}
 
 		fstr->bv_len = atmp.bv_len + vtmp.bv_len + STRLENOF( "(~=)" );
@@ -531,7 +547,7 @@ rwm_int_filter_map_rewrite(
 		if ( map_attr_value( dc, &f->f_sub_desc, &atmp,
 					NULL, NULL, RWM_MAP ) )
 		{
-			return -1;
+			goto computed;
 		}
 
 		/* cannot be a DN ... */
@@ -593,7 +609,7 @@ rwm_int_filter_map_rewrite(
 		if ( map_attr_value( dc, &f->f_desc, &atmp,
 					NULL, NULL, RWM_MAP ) )
 		{
-			return -1;
+			goto computed;
 		}
 
 		fstr->bv_len = atmp.bv_len + STRLENOF( "(=*)" );
@@ -614,11 +630,13 @@ rwm_int_filter_map_rewrite(
 			f->f_choice == LDAP_FILTER_OR ? '|' : '!' );
 
 		for ( p = f->f_list; p != NULL; p = p->f_next ) {
+			int	rc;
+
 			len = fstr->bv_len;
 
-			if ( rwm_int_filter_map_rewrite( dc, p, &vtmp ) )
-			{
-				return -1;
+			rc = rwm_int_filter_map_rewrite( op, dc, p, &vtmp );
+			if ( rc != LDAP_SUCCESS ) {
+				return rc;
 			}
 			
 			fstr->bv_len += vtmp.bv_len;
@@ -637,7 +655,7 @@ rwm_int_filter_map_rewrite(
 			if ( map_attr_value( dc, &f->f_mr_desc, &atmp,
 						&f->f_mr_value, &vtmp, RWM_MAP ) )
 			{
-				return -1;
+				goto computed;
 			}
 
 		} else {
@@ -662,26 +680,39 @@ rwm_int_filter_map_rewrite(
 		break;
 	}
 
+	case 0:
+computed:;
+		filter_free_x( op, f );
+		f->f_choice = SLAPD_FILTER_COMPUTED;
+		f->f_result = SLAPD_COMPARE_UNDEFINED;
+		/* fallthru */
+
 	case SLAPD_FILTER_COMPUTED:
 		switch ( f->f_result ) {
 		case LDAP_COMPARE_FALSE:
-			tmp = ber_bvfalse;
+		/* FIXME: treat UNDEFINED as FALSE */
+		case SLAPD_COMPARE_UNDEFINED:
+			if ( dc->rwmap->rwm_flags & RWM_F_SUPPORT_T_F ) {
+				tmp = &ber_bvtf_false;
+				break;
+			}
+			tmp = &ber_bvfalse;
 			break;
 
 		case LDAP_COMPARE_TRUE:
-			tmp = ber_bvtrue;
-			break;
-			
-		case SLAPD_COMPARE_UNDEFINED:
-			tmp = ber_bvundefined;
+			if ( dc->rwmap->rwm_flags & RWM_F_SUPPORT_T_F ) {
+				tmp = &ber_bvtf_true;
+				break;
+			}
+			tmp = &ber_bvtrue;
 			break;
 			
 		default:
-			tmp = ber_bverror;
+			tmp = &ber_bverror;
 			break;
 		}
 
-		ber_dupbv( fstr, &tmp );
+		ber_dupbv( fstr, tmp );
 		break;
 		
 	default:
@@ -689,23 +720,24 @@ rwm_int_filter_map_rewrite(
 		break;
 	}
 
-	return 0;
+	return LDAP_SUCCESS;
 }
 
 int
 rwm_filter_map_rewrite(
-		dncookie		*dc,
-		Filter			*f,
-		struct berval		*fstr )
+	Operation		*op,
+	dncookie		*dc,
+	Filter			*f,
+	struct berval		*fstr )
 {
 	int		rc;
 	dncookie 	fdc;
 	struct berval	ftmp;
 
-	rc = rwm_int_filter_map_rewrite( dc, f, fstr );
+	rc = rwm_int_filter_map_rewrite( op, dc, f, fstr );
 
 #ifdef ENABLE_REWRITE
-	if ( rc != LDAP_SUCCESS ) {
+	if ( rc != 0 ) {
 		return rc;
 	}
 
@@ -749,8 +781,8 @@ rwm_filter_map_rewrite(
 		rc = LDAP_OTHER;
 		break;
 	}
-
 #endif /* ENABLE_REWRITE */
+
 	return rc;
 }
 
@@ -779,7 +811,7 @@ rwm_referral_rewrite(
 	struct berval		dn = BER_BVNULL,
 				ndn = BER_BVNULL;
 
-	assert( a_vals );
+	assert( a_vals != NULL );
 
 	/*
 	 * Rewrite the dn if needed
@@ -806,7 +838,8 @@ rwm_referral_rewrite(
 	}
 
 	for ( i = 0; !BER_BVISNULL( &a_vals[i] ); i++ ) {
-		struct berval	olddn, oldval;
+		struct berval	olddn = BER_BVNULL,
+				oldval;
 		int		rc;
 		LDAPURLDesc	*ludp;
 
@@ -1049,8 +1082,7 @@ rwm_dnattr_rewrite(
 int
 rwm_referral_result_rewrite(
 	dncookie		*dc,
-	BerVarray		a_vals
-)
+	BerVarray		a_vals )
 {
 	int		i, last;
 
@@ -1058,7 +1090,8 @@ rwm_referral_result_rewrite(
 	last--;
 
 	for ( i = 0; !BER_BVISNULL( &a_vals[i] ); i++ ) {
-		struct berval	dn, olddn;
+		struct berval	dn,
+				olddn = BER_BVNULL;
 		int		rc;
 		LDAPURLDesc	*ludp;
 
@@ -1127,8 +1160,7 @@ rwm_referral_result_rewrite(
 int
 rwm_dnattr_result_rewrite(
 	dncookie		*dc,
-	BerVarray		a_vals
-)
+	BerVarray		a_vals )
 {
 	int		i, last;
 
@@ -1169,6 +1201,16 @@ rwm_dnattr_result_rewrite(
 	return 0;
 }
 
+void
+rwm_mapping_dst_free( void *v_mapping )
+{
+	struct ldapmapping *mapping = v_mapping;
+
+	if ( BER_BVISEMPTY( &mapping[0].m_dst ) ) {
+		rwm_mapping_free( &mapping[ -1 ] );
+	}
+}
+
 void
 rwm_mapping_free( void *v_mapping )
 {
