@@ -1,5 +1,5 @@
 /* slapi_overlay.c - SLAPI overlay */
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_overlay.c,v 1.40.2.1 2006/05/15 17:04:44 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_overlay.c,v 1.40.2.2 2006/05/17 15:27:21 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2001-2006 The OpenLDAP Foundation.
@@ -97,12 +97,14 @@ slapi_over_compute_output(
 {
 	Attribute		**a;
 	AttributeDescription	*desc;
-	SlapReply		*rs = (SlapReply *)c->cac_private;
+	SlapReply		*rs;
 
 	if ( c == NULL || attribute == NULL || entry == NULL ) {
 		return 0;
 	}
 
+	rs = (SlapReply *)c->cac_private;
+
 	assert( rs->sr_entry == entry );
 
 	desc = attribute->a_desc;
