@@ -1,8 +1,8 @@
 /* str2filter.c - parse an RFC 2554 string filter */
-/* $OpenLDAP: pkg/ldap/servers/slapd/str2filter.c,v 1.40 2004/11/26 09:10:29 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/str2filter.c,v 1.43 2006/01/03 22:12:16 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,12 +34,6 @@
 
 #include "slap.h"
 
-#if 0 /* unused */
-static char	*find_matching_paren( const char *s );
-#endif /* unused */
-static Filter	*str2list( const char *str, long unsigned int ftype);
-static Filter	*str2simple( const char *str);
-static int	str2subvals( const char *val, Filter *f);
 
 Filter *
 str2filter_x( Operation *op, const char *str )
